package com.tofly.security.component.token;

import com.tofly.security.Authority;
import com.tofly.security.SecurityUser;
import com.tofly.security.UserPrincipal;
import com.tofly.security.component.JwtSettings;
import com.tofly.security.component.exception.JwtExpiredTokenException;
import com.tofly.security.id.RoleEnum;
import com.tofly.security.id.UserId;
import com.tofly.security.model.JwtToken;
import io.jsonwebtoken.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.stereotype.Component;

import java.time.ZonedDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class JwtTokenFactory {

    private static final String SCOPES = "scopes";
    private static final String USER_ID = "userId";
    private static final String FIRST_NAME = "firstName";
    private static final String LAST_NAME = "lastName";
    private static final String ENABLED = "enabled";
    private static final String IS_PUBLIC = "isPublic";
    //    private static final String TENANT_ID = "tenantId";
//    private static final String CUSTOMER_ID = "customerId";
    private static final String ROLE = "role";
    private static final String CURRENT_USER_ID = "currentUserId";
    private static final String CURRENT_TENANT_ID = "currentTenantId";
    private static final String PLATFORM_USER_NAME = "platformUserName";

    private final JwtSettings settings;

    @Autowired
    public JwtTokenFactory(JwtSettings settings) {
        this.settings = settings;
    }

    /**
     * Factory method for issuing new JWT Tokens.
     */
    public AccessJwtToken createAccessJwtToken(SecurityUser securityUser) {
//        if (StringUtils.isBlank(securityUser.getEmail()))
//            throw new IllegalArgumentException("Cannot create JWT Token without username/email");
//
//        if (securityUser.getAuthority() == null)
//            throw new IllegalArgumentException("User doesn't have any privileges");

        UserPrincipal principal = securityUser.getUserPrincipal();
        String subject = principal.getValue();
        Claims claims = Jwts.claims().setSubject(subject);
        claims.put(SCOPES, securityUser.getAuthorities().stream().map(GrantedAuthority::getAuthority).collect(Collectors.toList()));
        claims.put(USER_ID, securityUser.getId().getId().toString());
        claims.put(PLATFORM_USER_NAME, securityUser.getName());
//        claims.put(FIRST_NAME, securityUser.getFirstName());
//        claims.put(LAST_NAME, securityUser.getLastName());
//        claims.put(ENABLED, securityUser.isEnabled());
//        claims.put(CURRENT_USER_ID,securityUser.getCurrentUserId());
//        claims.put(PLATFORM_USER_NAME,securityUser.getPlatformUserName());
        claims.put(ROLE, securityUser.getRoles());
//        claims.put(IS_PUBLIC, principal.getType() == UserPrincipal.Type.PUBLIC_ID);
//        if (securityUser.getTenantId() != null) {
//            claims.put(CURRENT_TENANT_ID,securityUser.getCurrentTenantId());
//            claims.put(TENANT_ID, securityUser.getTenantId().getId().toString());
//        }
//        if (securityUser.getCustomerId() != null) {
//            claims.put(CUSTOMER_ID, securityUser.getCustomerId().getId().toString());
//        }

        long now = System.currentTimeMillis();//当前毫秒
        long exp = now + settings.getTokenExpirationTime() * 1000 ;//加上配置文件中的过期时间(单位毫秒)


        Date nowDate = new Date();

        Date now_10 = new Date(nowDate.getTime() - 10 * 60 * 1000);

        String token = Jwts.builder()
                .setClaims(claims)
                .setIssuer(settings.getTokenIssuer())
                .setIssuedAt(now_10)
                .setExpiration(new Date(exp))
                .signWith(SignatureAlgorithm.HS512, settings.getTokenSigningKey())
                .compact();

        return new AccessJwtToken(token, claims);
    }

    public SecurityUser parseAccessJwtToken(RawAccessJwtToken rawAccessToken) {
        Jws<Claims> jwsClaims = parseTokenClaims(rawAccessToken);
        Claims claims = jwsClaims.getBody();
        String subject = claims.getSubject();
        @SuppressWarnings("unchecked")
//        List<String> scopes = claims.get(SCOPES, List.class);
//        if (scopes == null || scopes.isEmpty()) {
//            throw new IllegalArgumentException("JWT Token doesn't have any scopes");
//        }

        SecurityUser securityUser = new SecurityUser(UserId.fromString(claims.get(USER_ID, String.class)));
//        securityUser.setEmail(subject);
//        securityUser.setAuthority(Authority.parse(scopes.get(0)));
//        securityUser.setFirstName(claims.get(FIRST_NAME, String.class));
//        securityUser.setLastName(claims.get(LAST_NAME, String.class));
//        securityUser.setEnabled(claims.get(ENABLED, Boolean.class));
//        boolean isPublic = claims.get(IS_PUBLIC, Boolean.class);
        boolean isPublic = false;
        UserPrincipal principal = new UserPrincipal(isPublic ? UserPrincipal.Type.PUBLIC_ID : UserPrincipal.Type.USER_NAME, subject);
        securityUser.setUserPrincipal(principal);
        if(claims.get(ROLE) != null){
            securityUser.setRoles(new HashSet<>(((Collection<String>)claims.get(ROLE))));
        }
        if(null != securityUser.getRoles()){
            securityUser.setPtSysadmin(securityUser.getRoles().stream().anyMatch(role -> role.equals(RoleEnum.SYS_ADMIN.name())));
            securityUser.setPtTenantAdmin(securityUser.getRoles().stream().anyMatch(role -> role.equals(RoleEnum.TENANT_ADMIN.name())));
            securityUser.setPtAdmin(securityUser.getRoles().stream().anyMatch(role -> role.equals(RoleEnum.PLATFORM_ADMIN.name())));
        }
        securityUser.setCurrentUserId(claims.get(USER_ID, String.class));
//        securityUser.setPlatformUserName(claims.get(PLATFORM_USER_NAME,String.class));
//        String tenantId = claims.get(TENANT_ID, String.class);
//        if (tenantId != null) {
//            securityUser.setTenantId(TenantId.fromUUID(UUID.fromString(tenantId)));
//            securityUser.setCurrentTenantId(tenantId);
//        }
//        String customerId = claims.get(CUSTOMER_ID, String.class);
//        if (customerId != null) {
//            securityUser.setCustomerId(new CustomerId(UUID.fromString(customerId)));
//        }

        return securityUser;
    }

    public JwtToken createRefreshToken(SecurityUser securityUser) {
//        if (StringUtils.isBlank(securityUser.getEmail())) {
//            throw new IllegalArgumentException("Cannot create JWT Token without username/email");
//        }

        ZonedDateTime currentTime = ZonedDateTime.now();

        UserPrincipal principal = securityUser.getUserPrincipal();
        Claims claims = Jwts.claims().setSubject(principal.getValue());
        claims.put(SCOPES, Collections.singletonList(Authority.REFRESH_TOKEN.name()));
//        claims.put(USER_ID, securityUser.getId().getId().toString());
//        claims.put(IS_PUBLIC, principal.getType() == UserPrincipal.Type.PUBLIC_ID);

        String token = Jwts.builder()
                .setClaims(claims)
                .setIssuer(settings.getTokenIssuer())
                .setId(UUID.randomUUID().toString())
                .setIssuedAt(Date.from(currentTime.toInstant()))
                .setExpiration(Date.from(currentTime.plusSeconds(settings.getRefreshTokenExpTime()).toInstant()))
                .signWith(SignatureAlgorithm.HS512, settings.getTokenSigningKey())
                .compact();

        return new AccessJwtToken(token, claims);
    }

    public SecurityUser parseRefreshToken(RawAccessJwtToken rawAccessToken) {
        Jws<Claims> jwsClaims = parseTokenClaims(rawAccessToken);
        Claims claims = jwsClaims.getBody();
        String subject = claims.getSubject();
        @SuppressWarnings("unchecked")
        List<String> scopes = claims.get(SCOPES, List.class);
        if (scopes == null || scopes.isEmpty()) {
            throw new IllegalArgumentException("Refresh Token doesn't have any scopes");
        }
        if (!scopes.get(0).equals(Authority.REFRESH_TOKEN.name())) {
            throw new IllegalArgumentException("Invalid Refresh Token scope");
        }
        boolean isPublic = claims.get(IS_PUBLIC, Boolean.class);
        UserPrincipal principal = new UserPrincipal(isPublic ? UserPrincipal.Type.PUBLIC_ID : UserPrincipal.Type.USER_NAME, subject);
        SecurityUser securityUser = new SecurityUser();
        securityUser.setUserPrincipal(principal);
        return securityUser;
    }

    public Jws<Claims> parseTokenClaims(JwtToken token) {
        try {
            return Jwts.parser()
                    .setSigningKey(settings.getTokenSigningKey())
                    .parseClaimsJws(token.getToken());
        } catch (UnsupportedJwtException | MalformedJwtException | IllegalArgumentException | SignatureException ex) {
            log.debug("Invalid JWT Token", ex);
            throw new BadCredentialsException("Invalid JWT token: ", ex);
        } catch (ExpiredJwtException expiredEx) {
            log.debug("JWT Token is expired", expiredEx);
            throw new JwtExpiredTokenException(token, "JWT Token expired", expiredEx);
        }
    }
}