package com.tofly.toflygetway.filter;

import com.tofly.common.core.constant.SecurityConstants;
import com.tofly.common.core.util.StringUtil;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.UUID;
import java.util.stream.Collectors;

import static org.springframework.cloud.gateway.support.ServerWebExchangeUtils.GATEWAY_REQUEST_URL_ATTR;
import static org.springframework.cloud.gateway.support.ServerWebExchangeUtils.addOriginalRequestUrl;

/**
 * @author Administrator
 * @date 2019/9/17  13:54
 * 全局拦截器，作用所有的微服务
 * 1. 对请求头中参数进行处理 from 参数进行清洗
 * 2. 重写StripPrefix = 1,支持全局
 */
@Component
@Slf4j
public class ToflyRequestGlobalFilter implements GlobalFilter, Ordered {

    private static final String AUTHORIZATION_HEADER = "Authorization";
    private static final String X_AUTHORIZATION_HEADER = "X-Authorization";
    private static final String BEARER_PREFIX = "Bearer ";

    public static final String FORM_BASED_LOGIN_ENTRY_POINT = "/api/auth/login," +
           "/api/auth/dingLogin,"+
            "/yt/user/reset," +
            "/v2/api-docs," +
            "/file/loadImg," +
            "/file/download," +
            "/file/downloadFile," +
            "/file/loadAudio," +
            "/swagger2word/download,"+
            "/commonFile,"+
            "/api/commonFile/download";
    private static final String tokenSigningKey = "1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm";

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 1. 清洗请求头中from 参数
        ServerHttpRequest request = exchange.getRequest().mutate()
                .headers(httpHeaders -> httpHeaders.remove(SecurityConstants.FROM))
                .build();
        // 2. 重写StripPrefix
        addOriginalRequestUrl(exchange, request.getURI());
        String rawPath = request.getURI().getRawPath();
        String newPath = "/" + Arrays.stream(StringUtils.tokenizeToStringArray(rawPath, "/"))
                .skip(1L).collect(Collectors.joining("/"));
        ServerHttpRequest newRequest = request.mutate()
                .path(newPath)
                .build();
        exchange.getAttributes().put(GATEWAY_REQUEST_URL_ATTR, newRequest.getURI());
        String token = request.getHeaders().getFirst(AUTHORIZATION_HEADER) == null ?
                request.getHeaders().getFirst(X_AUTHORIZATION_HEADER) : request.getHeaders().getFirst(AUTHORIZATION_HEADER);
        if ((token != null && token.startsWith(BEARER_PREFIX))
                || FORM_BASED_LOGIN_ENTRY_POINT.indexOf(newPath) > -1||newPath.indexOf("/commonFile")>-1) {
            // 去掉"Bearer "前缀
            if (token != null && token.startsWith(BEARER_PREFIX)) {
                token = token.substring(BEARER_PREFIX.length());
            }
            if (FORM_BASED_LOGIN_ENTRY_POINT.indexOf(newPath) > -1 || validateToken(token)||newPath.indexOf("/commonFile")>-1) {
                return chain.filter(exchange.mutate()
                        .request(newRequest.mutate()
                                .build()).build());
            }
        }
        return unauthorized(exchange);
    }

    private Mono<Void> unauthorized(ServerWebExchange exchange) {
        exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
        exchange.getResponse().getHeaders().setContentType(MediaType.APPLICATION_JSON);

        String errorMessage = "Token无效";
        byte[] errorMessageBytes = errorMessage.getBytes(StandardCharsets.UTF_8);
        DataBuffer buffer = exchange.getResponse().bufferFactory().wrap(errorMessageBytes);
        return exchange.getResponse().writeWith(Mono.just(buffer));
    }

    private boolean validateToken(String token) {
        try {
            if (!StringUtil.isEmpty(token)) {
                Claims claims = Jwts.parser()
                        .setSigningKey(tokenSigningKey)
                        .parseClaimsJws(token)
                        .getBody();
                String userId = claims.get("userId", String.class);
                if (!StringUtil.isEmpty(userId)) {
                    return true;
                }
            }
            return false;
        } catch (Exception ex) {
            return false;
        }
    }

    @Override
    public int getOrder() {
        return -100;
    }

}
