package com.tofly.base.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.tofly.base.service.security.jwt.JwtAuthenticationProvider;
import com.tofly.base.service.security.jwt.JwtTokenAuthenticationProcessingFilter;
import com.tofly.base.service.security.jwt.SkipPathRequestMatcher;
import com.tofly.base.service.security.jwt.extractor.TokenExtractor;
import com.tofly.base.service.security.rest.DingLoginAuthenticationProvider;
import com.tofly.base.service.security.rest.RestAuthenticationProvider;
import com.tofly.base.service.security.rest.RestDingLoginProcessingFilter;
import com.tofly.base.service.security.rest.RestLoginProcessingFilter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import javax.servlet.Filter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Configuration
@EnableWebSecurity
public class BaseSecurityConfig extends WebSecurityConfigurerAdapter {
    @Autowired
    @Qualifier("defaultAuthenticationSuccessHandler")
    private AuthenticationSuccessHandler successHandler;

    @Autowired
    @Qualifier("defaultAuthenticationFailureHandler")
    private AuthenticationFailureHandler failureHandler;

    @Autowired
    @Qualifier("jwtHeaderTokenExtractor")
    private TokenExtractor jwtHeaderTokenExtractor;


    public static final String FORM_BASED_LOGIN_ENTRY_POINT = "/api/auth/login";
    public static final String FORM_DING_LOGIN_ENTRY_POINT = "/api/auth/dingLogin";
    public static final String JWT_TOKEN_HEADER_PARAM = "X-Authorization";
    public static final String JWT_TOKEN_HEADER_PARAM_V2 = "Authorization";

    public static final String JWT_TOKEN_QUERY_PARAM = "token";

    public static final String[] TOFLY_NOT_AUTH_API = new String[] {
            "/api/yt/auth/code/login", "/api/yt/third/bind","/api/auth/login",
            "/api/yt/third/login/*", "/api/yt/third/login/id/*", "/api/yt/third/authorize",
            "/api/yt/platform/get", "/api/yt/app_design/get", "/api/yt/noauth/**",
            "/api/yt/device/list", "/api/yt/kaptcha/**", "/api/yt/device-discovery/save",
            "/api/yt/user/reset","/api/yt/role/**","/api/yt/dict_item/**", "/error",
            "/v2/api-docs","/api/yt/user/info/**","/api/yt/dict_item/itemInfo/**",
            "/api/yt/user/getUserIdByRealName",
            "/api/yt/organization/getChildrenId",
            "/file/download/**,/commonFile/**",
            "/api/yt/role/**",
            "/report-service/**",
            "/**/jmreport/**"
    };

    public static final String WEBJARS_ENTRY_POINT = "/webjars/**";

    public static final String DEVICE_API_ENTRY_POINT = "/api/v1/**";

    public static final String PUBLIC_LOGIN_ENTRY_POINT = "/api/auth/login/public";
    public static final String TOKEN_REFRESH_ENTRY_POINT = "/api/auth/token";
    protected static final String[] NON_TOKEN_BASED_AUTH_ENTRY_POINTS = new String[]{"/index.html", "/assets/**", "/static/**", "/api/noauth/**", "/webjars/**", "/api/license/**", "/resources/**"};
    public static final String TOKEN_BASED_AUTH_ENTRY_POINT = "/api/**";
    public static final String WS_TOKEN_BASED_AUTH_ENTRY_POINT = "/api/ws/**";

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private RestAuthenticationProvider restAuthenticationProvider;
    @Autowired
    private DingLoginAuthenticationProvider dingLoginAuthenticationProvider;
    @Autowired
    private JwtAuthenticationProvider jwtAuthenticationProvider;
    

//   @Autowired
//    private RefreshTokenProcessingFilter refreshTokenAuthenticationProvider;

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http.headers().cacheControl().and().frameOptions().disable()
                .and()
                .cors()
                .and()
                .csrf().disable()
                .exceptionHandling()
                .and()
                .sessionManagement()
                .sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                .authorizeRequests()
                .antMatchers(TOFLY_NOT_AUTH_API).permitAll() // NOT_AUTH end-point
                .antMatchers(FORM_BASED_LOGIN_ENTRY_POINT).permitAll() // Login end-point
                .antMatchers(FORM_DING_LOGIN_ENTRY_POINT).permitAll() // DING login end-point
                .anyRequest()
                .authenticated()
                .and()
                .httpBasic().disable()
                .addFilterBefore(buildDingLoginProcessingFilter(), UsernamePasswordAuthenticationFilter.class)
                .addFilterBefore(buildRestLoginProcessingFilter(), UsernamePasswordAuthenticationFilter.class)
                .addFilterBefore(buildJwtTokenAuthenticationProcessingFilter(), UsernamePasswordAuthenticationFilter.class);
                
    }

    private Filter buildDingLoginProcessingFilter() {

        RestDingLoginProcessingFilter filter = new RestDingLoginProcessingFilter(FORM_DING_LOGIN_ENTRY_POINT, successHandler, failureHandler, objectMapper);
        filter.setAuthenticationManager(this.authenticationManager);
        return filter;
    }


    @Bean
    protected RestLoginProcessingFilter buildRestLoginProcessingFilter() throws Exception {
        RestLoginProcessingFilter filter = new RestLoginProcessingFilter(FORM_BASED_LOGIN_ENTRY_POINT, successHandler, failureHandler, objectMapper);
        filter.setAuthenticationManager(this.authenticationManager);
        return filter;
    }

    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    protected JwtTokenAuthenticationProcessingFilter buildJwtTokenAuthenticationProcessingFilter() throws Exception {
        List<String> pathsToSkip = new ArrayList<>(Arrays.asList(NON_TOKEN_BASED_AUTH_ENTRY_POINTS));
        pathsToSkip.addAll(Arrays.asList(WS_TOKEN_BASED_AUTH_ENTRY_POINT, TOKEN_REFRESH_ENTRY_POINT, FORM_BASED_LOGIN_ENTRY_POINT,
                PUBLIC_LOGIN_ENTRY_POINT, DEVICE_API_ENTRY_POINT, WEBJARS_ENTRY_POINT));

        //toflylinks function
        pathsToSkip.addAll(Arrays.asList(TOFLY_NOT_AUTH_API));

        SkipPathRequestMatcher matcher = new SkipPathRequestMatcher(pathsToSkip, TOKEN_BASED_AUTH_ENTRY_POINT);
        JwtTokenAuthenticationProcessingFilter filter
                = new JwtTokenAuthenticationProcessingFilter(failureHandler, jwtHeaderTokenExtractor, matcher);
        filter.setAuthenticationManager(this.authenticationManager);
        return filter;
    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) {
        auth.authenticationProvider(dingLoginAuthenticationProvider);
        auth.authenticationProvider(restAuthenticationProvider);
        auth.authenticationProvider(jwtAuthenticationProvider);

        //auth.authenticationProvider(refreshTokenAuthenticationProvider);
    }
}