/**
 * Copyright © 2016-2022 The Thingsboard Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tofly.base.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.tofly.base.entity.user.User;
import com.tofly.base.server.common.data.EntityType;
import com.tofly.base.server.common.data.HasName;
import com.tofly.base.server.common.data.constant.ModelConstants;
import com.tofly.base.server.common.data.core.exception.*;
import com.tofly.base.server.common.data.dto.UserDTO;
import com.tofly.base.server.common.data.enums.ActionType;
import com.tofly.base.server.common.data.id.*;
import com.tofly.base.server.common.data.page.PageLink;
import com.tofly.base.server.common.data.page.SortOrder;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.service.security.permission.AccessControlService;
import com.tofly.base.service.security.permission.Operation;
import com.tofly.base.service.security.permission.Resource;
import com.tofly.base.service.security.rest.RestAuthenticationProvider;
import com.tofly.base.service.user.CustomerService;
import com.tofly.base.service.user.TkUserService;
import com.tofly.base.service.user.UserService;
import jakarta.mail.MessagingException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.ExceptionHandler;

import javax.servlet.http.HttpServletResponse;
import java.util.Optional;
import java.util.UUID;





@Slf4j
public abstract class BaseController {

    /*Swagger UI description*/
    private static final ObjectMapper json = new ObjectMapper();

    @Autowired
    private ThingsboardErrorResponseHandler errorResponseHandler;
    
    @Autowired
    protected UserService userService;
    
    @Autowired
    protected AccessControlService accessControlService;
    
    @Autowired
    protected CustomerService customerService;
    
    @Autowired
    protected RestAuthenticationProvider restAuthenticationProvider;
    
    
    @Autowired
    protected TkUserService  tkUserService;
    
//    @Autowired
//    protected EntityActionService entityActionService;

   
//    @Autowired
//    protected OAuth2Service oAuth2Service;
//
//    @Autowired
//    protected OAuth2ConfigTemplateService oAuth2ConfigTemplateService;

//    @Autowired
//    protected ComponentDiscoveryService componentDescriptorService;
//
//    @Autowired
//    protected RuleChainService ruleChainService;
//
//    @Autowired
//    protected TbClusterService tbClusterService;

//    @Autowired
//    protected RelationService relationService;
//
//    @Autowired
//    protected AuditLogService auditLogService;

//    @Autowired
//    protected DeviceStateService deviceStateService;
//
//    @Autowired
//    protected EntityViewService entityViewService;
//
//    @Autowired
//    protected TelemetrySubscriptionService tsSubService;

//    @Autowired
//    protected AttributesService attributesService;

//    @Autowired
//    protected ClaimDevicesService claimDevicesService;
//
//    @Autowired
//    protected PartitionService partitionService;
//
//    @Autowired
//    protected TbResourceService resourceService;
//
//    @Autowired
//    protected OtaPackageService otaPackageService;
//
//    @Autowired
//    protected OtaPackageStateService otaPackageStateService;
//
//    @Autowired
//    protected RpcService rpcService;
//
//    @Autowired
//    protected TbQueueProducerProvider producerProvider;
//
//    @Autowired
//    protected TbTenantProfileCache tenantProfileCache;

//    @Autowired
//    protected TbDeviceProfileCache deviceProfileCache;
//
//    @Autowired(required = false)
//    protected EdgeService edgeService;
//
//    @Autowired(required = false)
//    protected EdgeNotificationService edgeNotificationService;
//
//    @Autowired(required = false)
//    protected EdgeRpcService edgeGrpcService;
//
    

//    @Value("${server.log_controller_error_stack_trace}")
//    @Getter
//    private boolean logControllerErrorStackTrace;

//    @Value("${edges.enabled}")
//    @Getter
//    protected boolean edgesEnabled;

    @ExceptionHandler(ThingsboardException.class)
    public void handleThingsboardException(ThingsboardException ex, HttpServletResponse response) {
        errorResponseHandler.handle(ex, response);
    }

    protected ThingsboardException handleException(Exception exception) {
        return handleException(exception, true);
    }

    private ThingsboardException handleException(Exception exception, boolean logException) {
//        if (logException && logControllerErrorStackTrace) {
//            log.error("Error [{}]", exception.getMessage(), exception);
//        }

        String cause = "";
        if (exception.getCause() != null) {
            cause = exception.getCause().getClass().getCanonicalName();
        }

        if (exception instanceof ThingsboardException) {
            return (ThingsboardException) exception;
        } else if (exception instanceof IllegalArgumentException || exception instanceof IncorrectParameterException
                || exception instanceof DataValidationException || cause.contains("IncorrectParameterException")) {
            return new ThingsboardException(exception.getMessage(), ThingsboardErrorCode.BAD_REQUEST_PARAMS);
        } else if (exception instanceof MessagingException) {
            return new ThingsboardException("Unable to send mail: " + exception.getMessage(), ThingsboardErrorCode.GENERAL);
        } else {
            return new ThingsboardException(exception.getMessage(), exception, ThingsboardErrorCode.GENERAL);
        }
    }

    public <T> T checkNotNull(T reference) throws ThingsboardException {
        return checkNotNull(reference, "Requested item wasn't found!");
    }

    <T> T checkNotNull(T reference, String notFoundMessage) throws ThingsboardException {
        if (reference == null) {
            throw new ThingsboardException(notFoundMessage, ThingsboardErrorCode.ITEM_NOT_FOUND);
        }
        return reference;
    }

    <T> T checkNotNull(Optional<T> reference) throws ThingsboardException {
        return checkNotNull(reference, "请求的内容不存在!");
    }

    <T> T checkNotNull(Optional<T> reference, String notFoundMessage) throws ThingsboardException {
        if (reference.isPresent()) {
            return reference.get();
        } else {
            throw new ThingsboardException(notFoundMessage, ThingsboardErrorCode.ITEM_NOT_FOUND);
        }
    }

    protected void checkParameter(String name, String param) throws ThingsboardException {
        if (StringUtils.isEmpty(param)) {
            throw new ThingsboardException("参数【 " + name + "】不能为空!", ThingsboardErrorCode.BAD_REQUEST_PARAMS);
        }
    }

    void checkArrayParameter(String name, String[] params) throws ThingsboardException {
        if (params == null || params.length == 0) {
            throw new ThingsboardException("参数【 " + name + "】不能为空!", ThingsboardErrorCode.BAD_REQUEST_PARAMS);
        } else {
            for (String param : params) {
                checkParameter(name, param);
            }
        }
    }

    protected UUID toUUID(String id) {
        return UUID.fromString(id);
    }

    protected PageLink createPageLink(int pageSize, int page, String textSearch, String sortProperty, String sortOrder) throws ThingsboardException {
        if (!StringUtils.isEmpty(sortProperty)) {
            SortOrder.Direction direction = SortOrder.Direction.ASC;
            if (!StringUtils.isEmpty(sortOrder)) {
                try {
                    direction = SortOrder.Direction.valueOf(sortOrder.toUpperCase());
                } catch (IllegalArgumentException e) {
                    throw new ThingsboardException("不支持的排序规则【" + sortOrder + "】!只支持 'ASC' 或 'DESC'.", ThingsboardErrorCode.BAD_REQUEST_PARAMS);
                }
            }
            SortOrder sort = new SortOrder(sortProperty, direction);
            return new PageLink(pageSize, page, textSearch, sort);
        } else {
            return new PageLink(pageSize, page, textSearch);
        }
    }

//    protected TimePageLink createTimePageLink(int pageSize, int page, String textSearch,
//                                    String sortProperty, String sortOrder, Long startTime, Long endTime) throws ThingsboardException {
//        PageLink pageLink = this.createPageLink(pageSize, page, textSearch, sortProperty, sortOrder);
//        return new TimePageLink(pageLink, startTime, endTime);
//    }

    protected SecurityUser getCurrentUser() throws ThingsboardException {
        final SecurityContext context = SecurityContextHolder.getContext();
        Authentication authentication = context.getAuthentication();
    	SecurityUser user = (SecurityUser) authentication.getPrincipal();
    	UserDTO userDTO = tkUserService.findUserInfoById(user.getId().toString());
    	
    	//UserDTO userDTO = tkUserService.getUserByPhoneNumberOrUsername(principalValue);
    	user.setTenantId(TenantId.fromString(userDTO.getTenantId()));
    	user.setCurrentTenantId(userDTO.getTenantId().toString());
    	user.setPlatformUserName(userDTO.getUsername());
        
        if (authentication != null && authentication.getPrincipal() instanceof SecurityUser) {
            return user;
        } else {
            throw new ThingsboardException("你没有权限执行改操作!", ThingsboardErrorCode.AUTHENTICATION);
        }
    }

//    protected Tenant checkTenantId(TenantId tenantId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(tenantId, INCORRECT_TENANT_ID + tenantId);
//            Tenant tenant = tenantService.findTenantById(tenantId);
//            checkNotNull(tenant, "ID【" + tenantId + "】相关的租户不存在");
//            accessControlService.checkPermission(getCurrentUser(), Resource.TENANT, operation, tenantId, tenant);
//            return tenant;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    TenantInfo checkTenantInfoId(TenantId tenantId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(tenantId, INCORRECT_TENANT_ID + tenantId);
//            TenantInfo tenant = tenantService.findTenantInfoById(tenantId);
//            checkNotNull(tenant, "ID【" + tenantId + "】相关的租户不存在");
//            accessControlService.checkPermission(getCurrentUser(), Resource.TENANT, operation, tenantId, tenant);
//            return tenant;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    TenantProfile checkTenantProfileId(TenantProfileId tenantProfileId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(tenantProfileId, "错误的租户配置ID " + tenantProfileId);
//            TenantProfile tenantProfile = tenantProfileService.findTenantProfileById(getTenantId(), tenantProfileId);
//            checkNotNull(tenantProfile, "ID【" + tenantProfileId + "】相关的租户配置不存在");
//            accessControlService.checkPermission(getCurrentUser(), Resource.TENANT_PROFILE, operation);
//            return tenantProfile;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

    protected TenantId getTenantId() throws ThingsboardException {
        return getCurrentUser().getTenantId();
    }

//    protected Customer checkCustomerId(CustomerId customerId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(customerId, "客户ID不正确 " + customerId);
//            Customer customer = customerService.findCustomerById(getTenantId(), customerId);
//            checkNotNull(customer, "ID【" + customerId + "】相关的客户不存在");
//            accessControlService.checkPermission(getCurrentUser(), Resource.CUSTOMER, operation, customerId, customer);
//            return customer;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

    protected User checkUserId(UserId userId, Operation operation) throws ThingsboardException {
        try {
        	com.tofly.base.service.validator.Validator.validateId(userId, "用户ID不正确 " + userId);
            User user = userService.findUserById(getCurrentUser().getTenantId(), userId);
            checkNotNull(user, "ID【" + userId + "】相关的用户不存在");
            accessControlService.checkPermission(getCurrentUser(), Resource.USER, operation, userId, user);
            return user;
        } catch (Exception e) {
            throw handleException(e, false);
        }
    }

//    protected <I extends EntityId, T extends HasTenantId> void checkEntity(I entityId, T entity, Resource resource) throws ThingsboardException {
//        if (entityId == null) {
//            accessControlService
//                    .checkPermission(getCurrentUser(), resource, Operation.CREATE, null, entity);
//        } else {
//            checkEntityId(entityId, Operation.WRITE);
//        }
//    }

//    protected void checkEntityId(EntityId entityId, Operation operation) throws ThingsboardException {
//        try {
//            if (entityId == null) {
//                throw new ThingsboardException("参数entityId不能为空!", ThingsboardErrorCode.BAD_REQUEST_PARAMS);
//            }
//            validateId(entityId.getId(), "entityId不正确 " + entityId);
//            switch (entityId.getEntityType()) {
//                case ALARM:
//                    checkAlarmId(new AlarmId(entityId.getId()), operation);
//                    return;
//                case DEVICE:
//                    checkDeviceId(new DeviceId(entityId.getId()), operation);
//                    return;
//                case DEVICE_PROFILE:
//                    checkDeviceProfileId(new DeviceProfileId(entityId.getId()), operation);
//                    return;
//                case CUSTOMER:
//                    checkCustomerId(new CustomerId(entityId.getId()), operation);
//                    return;
//                case TENANT:
//                    checkTenantId(TenantId.fromUUID(entityId.getId()), operation);
//                    return;
//                case TENANT_PROFILE:
//                    checkTenantProfileId(new TenantProfileId(entityId.getId()), operation);
//                    return;
//                case RULE_CHAIN:
//                    checkRuleChain(new RuleChainId(entityId.getId()), operation);
//                    return;
//                case RULE_NODE:
//                    checkRuleNode(new RuleNodeId(entityId.getId()), operation);
//                    return;
//                case ASSET:
//                    checkAssetId(new AssetId(entityId.getId()), operation);
//                    return;
//                case DASHBOARD:
//                    checkDashboardId(new DashboardId(entityId.getId()), operation);
//                    return;
//                case USER:
//                    checkUserId(new UserId(entityId.getId()), operation);
//                    return;
//                case ENTITY_VIEW:
//                    checkEntityViewId(new EntityViewId(entityId.getId()), operation);
//                    return;
//                case EDGE:
//                    checkEdgeId(new EdgeId(entityId.getId()), operation);
//                    return;
//                case WIDGETS_BUNDLE:
//                    checkWidgetsBundleId(new WidgetsBundleId(entityId.getId()), operation);
//                    return;
//                case WIDGET_TYPE:
//                    checkWidgetTypeId(new WidgetTypeId(entityId.getId()), operation);
//                    return;
//                case TB_RESOURCE:
//                    checkResourceId(new TbResourceId(entityId.getId()), operation);
//                    return;
//                case OTA_PACKAGE:
//                    checkOtaPackageId(new OtaPackageId(entityId.getId()), operation);
//                    return;
//                default:
//                    throw new IllegalArgumentException("不支持的资源类型: " + entityId.getEntityType());
//            }
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    public Device checkDeviceId(DeviceId deviceId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(deviceId, "deviceId不正确 " + deviceId);
//            Device device = deviceService.findDeviceById(getCurrentUser().getTenantId(), deviceId);
//            checkNotNull(device, "ID【" + deviceId + "】相关的设备不存在");
//            accessControlService.checkPermission(getCurrentUser(), Resource.DEVICE, operation, deviceId, device);
//            return device;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    DeviceInfo checkDeviceInfoId(DeviceId deviceId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(deviceId, "deviceId不正确" + deviceId);
//            DeviceInfo device = deviceService.findDeviceInfoById(getCurrentUser().getTenantId(), deviceId);
//            checkNotNull(device, "ID【" + deviceId + "】相关的设备不存在");
//            accessControlService.checkPermission(getCurrentUser(), Resource.DEVICE, operation, deviceId, device);
//            return device;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    protected DeviceProfile checkDeviceProfileId(DeviceProfileId deviceProfileId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(deviceProfileId, "设备配置Id不正确 " + deviceProfileId);
//            DeviceProfile deviceProfile = deviceProfileService.findDeviceProfileById(getCurrentUser().getTenantId(), deviceProfileId);
//            checkNotNull(deviceProfile, "ID【" + deviceProfileId + "】相关的设备配置不存在");
//            accessControlService.checkPermission(getCurrentUser(), Resource.DEVICE_PROFILE, operation, deviceProfileId, deviceProfile);
//            return deviceProfile;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    protected EntityView checkEntityViewId(EntityViewId entityViewId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(entityViewId, "Incorrect entityViewId " + entityViewId);
//            EntityView entityView = entityViewService.findEntityViewById(getCurrentUser().getTenantId(), entityViewId);
//            checkNotNull(entityView, "Entity view with id [" + entityViewId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.ENTITY_VIEW, operation, entityViewId, entityView);
//            return entityView;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    EntityViewInfo checkEntityViewInfoId(EntityViewId entityViewId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(entityViewId, "Incorrect entityViewId " + entityViewId);
//            EntityViewInfo entityView = entityViewService.findEntityViewInfoById(getCurrentUser().getTenantId(), entityViewId);
//            checkNotNull(entityView, "Entity view with id [" + entityViewId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.ENTITY_VIEW, operation, entityViewId, entityView);
//            return entityView;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    Asset checkAssetId(AssetId assetId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(assetId, "Incorrect assetId " + assetId);
//            Asset asset = assetService.findAssetById(getCurrentUser().getTenantId(), assetId);
//            checkNotNull(asset, "Asset with id [" + assetId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.ASSET, operation, assetId, asset);
//            return asset;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    AssetInfo checkAssetInfoId(AssetId assetId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(assetId, "Incorrect assetId " + assetId);
//            AssetInfo asset = assetService.findAssetInfoById(getCurrentUser().getTenantId(), assetId);
//            checkNotNull(asset, "Asset with id [" + assetId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.ASSET, operation, assetId, asset);
//            return asset;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    Alarm checkAlarmId(AlarmId alarmId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(alarmId, "Incorrect alarmId " + alarmId);
//            Alarm alarm = alarmService.findAlarmByIdAsync(getCurrentUser().getTenantId(), alarmId).get();
//            checkNotNull(alarm, "Alarm with id [" + alarmId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.ALARM, operation, alarmId, alarm);
//            return alarm;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    AlarmInfo checkAlarmInfoId(AlarmId alarmId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(alarmId, "Incorrect alarmId " + alarmId);
//            AlarmInfo alarmInfo = alarmService.findAlarmInfoByIdAsync(getCurrentUser().getTenantId(), alarmId).get();
//            checkNotNull(alarmInfo, "Alarm with id [" + alarmId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.ALARM, operation, alarmId, alarmInfo);
//            return alarmInfo;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    WidgetsBundle checkWidgetsBundleId(WidgetsBundleId widgetsBundleId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(widgetsBundleId, "Incorrect widgetsBundleId " + widgetsBundleId);
//            WidgetsBundle widgetsBundle = widgetsBundleService.findWidgetsBundleById(getCurrentUser().getTenantId(), widgetsBundleId);
//            checkNotNull(widgetsBundle, "Widgets bundle with id [" + widgetsBundleId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.WIDGETS_BUNDLE, operation, widgetsBundleId, widgetsBundle);
//            return widgetsBundle;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    WidgetTypeDetails checkWidgetTypeId(WidgetTypeId widgetTypeId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(widgetTypeId, "Incorrect widgetTypeId " + widgetTypeId);
//            WidgetTypeDetails widgetTypeDetails = widgetTypeService.findWidgetTypeDetailsById(getCurrentUser().getTenantId(), widgetTypeId);
//            checkNotNull(widgetTypeDetails, "Widget type with id [" + widgetTypeId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.WIDGET_TYPE, operation, widgetTypeId, widgetTypeDetails);
//            return widgetTypeDetails;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    Dashboard checkDashboardId(DashboardId dashboardId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(dashboardId, "Incorrect dashboardId " + dashboardId);
//            Dashboard dashboard = dashboardService.findDashboardById(getCurrentUser().getTenantId(), dashboardId);
//            checkNotNull(dashboard, "Dashboard with id [" + dashboardId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.DASHBOARD, operation, dashboardId, dashboard);
//            return dashboard;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    Edge checkEdgeId(EdgeId edgeId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(edgeId, "Incorrect edgeId " + edgeId);
//            Edge edge = edgeService.findEdgeById(getTenantId(), edgeId);
//            checkNotNull(edge, "Edge with id [" + edgeId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.EDGE, operation, edgeId, edge);
//            return edge;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    EdgeInfo checkEdgeInfoId(EdgeId edgeId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(edgeId, "Incorrect edgeId " + edgeId);
//            EdgeInfo edge = edgeService.findEdgeInfoById(getCurrentUser().getTenantId(), edgeId);
//            checkNotNull(edge, "Edge with id [" + edgeId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.EDGE, operation, edgeId, edge);
//            return edge;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    DashboardInfo checkDashboardInfoId(DashboardId dashboardId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(dashboardId, "Incorrect dashboardId " + dashboardId);
//            DashboardInfo dashboardInfo = dashboardService.findDashboardInfoById(getCurrentUser().getTenantId(), dashboardId);
//            checkNotNull(dashboardInfo, "Dashboard with id [" + dashboardId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.DASHBOARD, operation, dashboardId, dashboardInfo);
//            return dashboardInfo;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    ComponentDescriptor checkComponentDescriptorByClazz(String clazz) throws ThingsboardException {
//        try {
//            log.debug("[{}] Lookup component descriptor", clazz);
//            return checkNotNull(componentDescriptorService.getComponent(clazz));
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    List<ComponentDescriptor> checkComponentDescriptorsByType(ComponentType type, RuleChainType ruleChainType) throws ThingsboardException {
//        try {
//            log.debug("[{}] Lookup component descriptors", type);
//            return componentDescriptorService.getComponents(type, ruleChainType);
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    List<ComponentDescriptor> checkComponentDescriptorsByTypes(Set<ComponentType> types, RuleChainType ruleChainType) throws ThingsboardException {
//        try {
//            log.debug("[{}] Lookup component descriptors", types);
//            return componentDescriptorService.getComponents(types, ruleChainType);
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    protected RuleChain checkRuleChain(RuleChainId ruleChainId, Operation operation) throws ThingsboardException {
//        validateId(ruleChainId, "Incorrect ruleChainId " + ruleChainId);
//        RuleChain ruleChain = ruleChainService.findRuleChainById(getCurrentUser().getTenantId(), ruleChainId);
//        checkNotNull(ruleChain, "Rule chain with id [" + ruleChainId + "] is not found");
//        accessControlService.checkPermission(getCurrentUser(), Resource.RULE_CHAIN, operation, ruleChainId, ruleChain);
//        return ruleChain;
//    }
//
//    protected RuleNode checkRuleNode(RuleNodeId ruleNodeId, Operation operation) throws ThingsboardException {
//        validateId(ruleNodeId, "Incorrect ruleNodeId " + ruleNodeId);
//        RuleNode ruleNode = ruleChainService.findRuleNodeById(getTenantId(), ruleNodeId);
//        checkNotNull(ruleNode, "Rule node with id [" + ruleNodeId + "] is not found");
//        checkRuleChain(ruleNode.getRuleChainId(), operation);
//        return ruleNode;
//    }
//
//    TbResource checkResourceId(TbResourceId resourceId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(resourceId, "Incorrect resourceId " + resourceId);
//            TbResource resource = resourceService.findResourceById(getCurrentUser().getTenantId(), resourceId);
//            checkNotNull(resource, "Resource with id [" + resourceId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.TB_RESOURCE, operation, resourceId, resource);
//            return resource;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    TbResourceInfo checkResourceInfoId(TbResourceId resourceId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(resourceId, "Incorrect resourceId " + resourceId);
//            TbResourceInfo resourceInfo = resourceService.findResourceInfoById(getCurrentUser().getTenantId(), resourceId);
//            checkNotNull(resourceInfo, "Resource with id [" + resourceId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.TB_RESOURCE, operation, resourceId, resourceInfo);
//            return resourceInfo;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    OtaPackage checkOtaPackageId(OtaPackageId otaPackageId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(otaPackageId, "Incorrect otaPackageId " + otaPackageId);
//            OtaPackage otaPackage = otaPackageService.findOtaPackageById(getCurrentUser().getTenantId(), otaPackageId);
//            checkNotNull(otaPackage, "OTA package with id [" + otaPackageId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.OTA_PACKAGE, operation, otaPackageId, otaPackage);
//            return otaPackage;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }
//
//    OtaPackageInfo checkOtaPackageInfoId(OtaPackageId otaPackageId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(otaPackageId, "Incorrect otaPackageId " + otaPackageId);
//            OtaPackageInfo otaPackageIn = otaPackageService.findOtaPackageInfoById(getCurrentUser().getTenantId(), otaPackageId);
//            checkNotNull(otaPackageIn, "OTA package with id [" + otaPackageId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.OTA_PACKAGE, operation, otaPackageId, otaPackageIn);
//            return otaPackageIn;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

//    Rpc checkRpcId(RpcId rpcId, Operation operation) throws ThingsboardException {
//        try {
//            validateId(rpcId, "Incorrect rpcId " + rpcId);
//            Rpc rpc = rpcService.findById(getCurrentUser().getTenantId(), rpcId);
//            checkNotNull(rpc, "RPC with id [" + rpcId + "] is not found");
//            accessControlService.checkPermission(getCurrentUser(), Resource.RPC, operation, rpcId, rpc);
//            return rpc;
//        } catch (Exception e) {
//            throw handleException(e, false);
//        }
//    }

    @SuppressWarnings("unchecked")
    protected <I extends EntityId> I emptyId(EntityType entityType) {
        return (I) EntityIdFactory.getByTypeAndUuid(entityType, ModelConstants.NULL_UUID);
    }

    protected <E extends HasName, I extends EntityId> void logEntityAction(I entityId, E entity, CustomerId customerId,
                                                                           ActionType actionType, Exception e, Object... additionalInfo) throws ThingsboardException {
        logEntityAction(getCurrentUser(), entityId, entity, customerId, actionType, e, additionalInfo);
    }

    protected <E extends HasName, I extends EntityId> void logEntityAction(User user, I entityId, E entity, CustomerId customerId,
                                                                           ActionType actionType, Exception e, Object... additionalInfo) throws ThingsboardException {
        //entityActionService.logEntityAction(user, entityId, entity, customerId, actionType, e, additionalInfo);
    }


    public static Exception toException(Throwable error) {
        return error != null ? (Exception.class.isInstance(error) ? (Exception) error : new Exception(error)) : null;
    }

    protected <E extends HasName> String entityToStr(E entity) {
        try {
            return json.writeValueAsString(json.valueToTree(entity));
        } catch (JsonProcessingException e) {
            log.warn("[{}] Failed to convert entity to string!", entity, e);
        }
        return null;
    }

//    protected void sendRelationNotificationMsg(TenantId tenantId, EntityRelation relation, EdgeEventActionType action) {
//        try {
//            if (!relation.getFrom().getEntityType().equals(EntityType.EDGE) &&
//                    !relation.getTo().getEntityType().equals(EntityType.EDGE)) {
//                sendNotificationMsgToEdgeService(tenantId, null, null, json.writeValueAsString(relation), EdgeEventType.RELATION, action);
//            }
//        } catch (Exception e) {
//            log.warn("Failed to push relation to core: {}", relation, e);
//        }
//    }
//
//    protected void sendDeleteNotificationMsg(TenantId tenantId, EntityId entityId, List<EdgeId> edgeIds) {
//        sendDeleteNotificationMsg(tenantId, entityId, edgeIds, null);
//    }
//
//    protected void sendDeleteNotificationMsg(TenantId tenantId, EntityId entityId, List<EdgeId> edgeIds, String body) {
//        if (edgeIds != null && !edgeIds.isEmpty()) {
//            for (EdgeId edgeId : edgeIds) {
//                sendNotificationMsgToEdgeService(tenantId, edgeId, entityId, body, null, EdgeEventActionType.DELETED);
//            }
//        }
//    }
//
//    protected void sendAlarmDeleteNotificationMsg(TenantId tenantId, EntityId entityId, List<EdgeId> edgeIds, Alarm alarm) {
//        try {
//            sendDeleteNotificationMsg(tenantId, entityId, edgeIds, json.writeValueAsString(alarm));
//        } catch (Exception e) {
//            log.warn("Failed to push delete alarm msg to core: {}", alarm, e);
//        }
//    }
//
//    protected void sendEntityAssignToCustomerNotificationMsg(TenantId tenantId, EntityId entityId, CustomerId customerId, EdgeEventActionType action) {
//        try {
//            sendNotificationMsgToEdgeService(tenantId, null, entityId, json.writeValueAsString(customerId), null, action);
//        } catch (Exception e) {
//            log.warn("Failed to push assign/unassign to/from customer to core: {}", customerId, e);
//        }
//    }
//
//    protected void sendEntityNotificationMsg(TenantId tenantId, EntityId entityId, EdgeEventActionType action) {
//        sendNotificationMsgToEdgeService(tenantId, null, entityId, null, null, action);
//    }
//
//    protected void sendEntityAssignToEdgeNotificationMsg(TenantId tenantId, EdgeId edgeId, EntityId entityId, EdgeEventActionType action) {
//        sendNotificationMsgToEdgeService(tenantId, edgeId, entityId, null, null, action);
//    }
//
//    private void sendNotificationMsgToEdgeService(TenantId tenantId, EdgeId edgeId, EntityId entityId, String body, EdgeEventType type, EdgeEventActionType action) {
//        tbClusterService.sendNotificationMsgToEdgeService(tenantId, edgeId, entityId, body, type, action);
//    }
//
//    protected List<EdgeId> findRelatedEdgeIds(TenantId tenantId, EntityId entityId) {
//        if (!edgesEnabled) {
//            return null;
//        }
//        if (EntityType.EDGE.equals(entityId.getEntityType())) {
//            return Collections.singletonList(new EdgeId(entityId.getId()));
//        }
//        PageDataIterableByTenantIdEntityId<EdgeId> relatedEdgeIdsIterator =
//                new PageDataIterableByTenantIdEntityId<>(edgeService::findRelatedEdgeIdsByEntityId, tenantId, entityId, DEFAULT_PAGE_SIZE);
//        List<EdgeId> result = new ArrayList<>();
//        for (EdgeId edgeId : relatedEdgeIdsIterator) {
//            result.add(edgeId);
//        }
//        return result;
//    }
//
//    protected void processDashboardIdFromAdditionalInfo(ObjectNode additionalInfo, String requiredFields) throws ThingsboardException {
//        String dashboardId = additionalInfo.has(requiredFields) ? additionalInfo.get(requiredFields).asText() : null;
//        if (dashboardId != null && !dashboardId.equals("null")) {
//            if (dashboardService.findDashboardById(getTenantId(), new DashboardId(UUID.fromString(dashboardId))) == null) {
//                additionalInfo.remove(requiredFields);
//            }
//        }
//    }

    protected MediaType parseMediaType(String contentType) {
        try {
            return MediaType.parseMediaType(contentType);
        } catch (Exception e) {
            return MediaType.APPLICATION_OCTET_STREAM;
        }
    }

//    /**
//     * 构建设备配置的配置数据
//     * @param transportType            产品的通信协议
//     * @param deviceProfileData        空的设备配置数据
//     * @param scriptText               自定义数据协议的解析脚本
//     */
//    protected DeviceProfileData buildDeviceProfileData(String transportType,DeviceProfileData deviceProfileData,String scriptText) {
//        if(null == deviceProfileData){
//            deviceProfileData = new DeviceProfileData();
//        }
//        deviceProfileData.setConfiguration(new DefaultDeviceProfileConfiguration());
//        deviceProfileData.setProvisionConfiguration(new DisabledDeviceProfileProvisionConfiguration(null));
//
//        // 传输类型默认都是Default
//        if(transportType ==null || DeviceTransportType.DEFAULT.name().equals(transportType)){
//            deviceProfileData.setTransportConfiguration(new DefaultDeviceProfileTransportConfiguration());
//        }else if(DeviceTransportType.TCP.name().equals(transportType)){
//            TkTcpDeviceProfileTransportConfiguration tcpDeviceProfileTransportConfiguration = (TkTcpDeviceProfileTransportConfiguration) deviceProfileData.getTransportConfiguration();
//            tcpDeviceProfileTransportConfiguration.setScriptText(scriptText);
//            deviceProfileData.setTransportConfiguration(tcpDeviceProfileTransportConfiguration);
//        }else{
//            deviceProfileData.setTransportConfiguration(deviceProfileData.getTransportConfiguration());
//        }
//        return deviceProfileData;
//
//    }
//
//
//    /**
//     * 更新thingsboard的设备配置信息
//     *
//     * @param deviceProfile 设备配置
//     * @throws ThingsboardException
//     */
//    protected DeviceProfile updateTbDeviceProfile(DeviceProfile deviceProfile)
//            throws ThingsboardException {
//        boolean isFirmwareChanged = false;
//        boolean isSoftwareChanged = false;
//
//        DeviceProfile oldDeviceProfile =
//                deviceProfileService.findDeviceProfileById(getTenantId(), deviceProfile.getId());
//        if (!Objects.equals(deviceProfile.getFirmwareId(), oldDeviceProfile.getFirmwareId())) {
//            isFirmwareChanged = true;
//        }
//        if (!Objects.equals(deviceProfile.getSoftwareId(), oldDeviceProfile.getSoftwareId())) {
//            isSoftwareChanged = true;
//        }
//        if (FastIotConstants.ASSERT_DEFAULT_NAME.equals(oldDeviceProfile.getName())
//                && !Objects.equals(deviceProfile.getName(), oldDeviceProfile.getName())) {
//            throw new TkDataValidationException(ErrorMessage.ASSERT_DEFAULT_NAME_NO_CHANGED.getMessage());
//        }
//
//        DeviceProfile savedDeviceProfile =
//                checkNotNull(deviceProfileService.saveDeviceProfile(deviceProfile));
//
//        tbClusterService.onDeviceProfileChange(savedDeviceProfile, null);
//        tbClusterService.broadcastEntityStateChangeEvent(
//                deviceProfile.getTenantId(), savedDeviceProfile.getId(), ComponentLifecycleEvent.UPDATED);
//
//        logEntityAction(savedDeviceProfile.getId(), savedDeviceProfile, null, ActionType.UPDATED, null);
//
//        otaPackageStateService.update(savedDeviceProfile, isFirmwareChanged, isSoftwareChanged);
//
//        sendEntityNotificationMsg(
//                getTenantId(), savedDeviceProfile.getId(), EdgeEventActionType.UPDATED);
//        return savedDeviceProfile;
//    }
}
