package com.tofly.base.entity.user;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.JsonNode;
import com.tofly.base.server.common.data.HasCustomerId;
import com.tofly.base.server.common.data.HasName;
import com.tofly.base.server.common.data.HasTenantId;
import com.tofly.base.server.common.data.SearchTextBasedWithAdditionalInfo;
import com.tofly.base.server.common.data.dto.UserDetailsDTO;
import com.tofly.base.server.common.data.id.CustomerId;
import com.tofly.base.server.common.data.id.EntityId;
import com.tofly.base.server.common.data.id.TenantId;
import com.tofly.base.server.common.data.id.UserId;
import com.tofly.base.server.common.data.security.Authority;
import com.tofly.base.server.common.data.vaildation.Length;
import com.tofly.base.server.common.data.vaildation.NoXss;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;


@ApiModel
@EqualsAndHashCode(callSuper = true)
public class User extends SearchTextBasedWithAdditionalInfo<UserId>
    implements HasName, HasTenantId, HasCustomerId {

  private static final long serialVersionUID = 8250339805336035966L;

  private TenantId tenantId;
  private CustomerId customerId;
  private String email;
  private Authority authority;

  @NoXss
  @Length(fieldName = "first name")
  private String firstName;

  @NoXss
  @Length(fieldName = "last name")
  private String lastName;
  /** 平台的用户详情 */
  @Getter @Setter private UserDetailsDTO userDetailsDTO;

  public User() {
    super();
  }

  public User(UserId id) {
    super(id);
  }

  public User(User user) {
    super(user);
    this.tenantId = user.getTenantId();
    this.customerId = user.getCustomerId();
    this.email = user.getEmail();
    this.authority = user.getAuthority();
    this.firstName = user.getFirstName();
    this.lastName = user.getLastName();
  }

  @ApiModelProperty(
      position = 1,
      value =
          "JSON object with the User Id. "
              + "Specify this field to update the device. "
              + "Referencing non-existing User Id will cause error. "
              + "Omit this field to create new customer.")
  @Override
  public UserId getId() {
    return super.getId();
  }

  @ApiModelProperty(
      position = 2,
      value = "Timestamp of the user creation, in milliseconds",
      example = "1609459200000",
      readOnly = true)
  @Override
  public long getCreatedTime() {
    return super.getCreatedTime();
  }

  @ApiModelProperty(position = 3, value = "JSON object with the Tenant Id.", readOnly = true)
  public TenantId getTenantId() {
    return tenantId;
  }

  public void setTenantId(TenantId tenantId) {
    this.tenantId = tenantId;
  }

  @ApiModelProperty(position = 4, value = "JSON object with the Customer Id.", readOnly = true)
  public CustomerId getCustomerId() {
    return customerId;
  }

  public void setCustomerId(CustomerId customerId) {
    this.customerId = customerId;
  }

  @ApiModelProperty(
      position = 5,
      required = true,
      value = "Email of the user",
      example = "user@example.com")
  public String getEmail() {
    return email;
  }

  public void setEmail(String email) {
    this.email = email;
  }

  @ApiModelProperty(
      position = 6,
      readOnly = true,
      value = "Duplicates the email of the user, readonly",
      example = "user@example.com")
  @Override
  @JsonProperty(access = JsonProperty.Access.READ_ONLY)
  public String getName() {
    return email;
  }

  @ApiModelProperty(
      position = 7,
      required = true,
      value = "Authority",
      example = "SYS_ADMIN, TENANT_ADMIN or CUSTOMER_USER")
  public Authority getAuthority() {
    return authority;
  }

  public void setAuthority(Authority authority) {
    this.authority = authority;
  }

  @ApiModelProperty(
      position = 8,
      required = true,
      value = "First name of the user",
      example = "John")
  public String getFirstName() {
    return firstName;
  }

  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

  @ApiModelProperty(position = 9, required = true, value = "Last name of the user", example = "Doe")
  public String getLastName() {
    return lastName;
  }

  public void setLastName(String lastName) {
    this.lastName = lastName;
  }

  @ApiModelProperty(
      position = 10,
      value = "Additional parameters of the user",
      dataType = "com.fasterxml.jackson.databind.JsonNode")
  @Override
  public JsonNode getAdditionalInfo() {
    return super.getAdditionalInfo();
  }

  @Override
  public String getSearchText() {
    return getEmail();
  }

  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder();
    builder.append("User [tenantId=");
    builder.append(tenantId);
    builder.append(", customerId=");
    builder.append(customerId);
    builder.append(", email=");
    builder.append(email);
    builder.append(", authority=");
    builder.append(authority);
    builder.append(", firstName=");
    builder.append(firstName);
    builder.append(", lastName=");
    builder.append(lastName);
    builder.append(", additionalInfo=");
    builder.append(getAdditionalInfo());
    builder.append(", createdTime=");
    builder.append(createdTime);
    builder.append(", id=");
    builder.append(id);
    builder.append("]");
    return builder.toString();
  }

  @JsonIgnore
  public boolean isSystemAdmin() {
    return (tenantId == null || EntityId.NULL_UUID.equals(tenantId.getId()));
  }

  @JsonIgnore
  public boolean isTenantAdmin() {
    return !isSystemAdmin()
        && (customerId == null || EntityId.NULL_UUID.equals(customerId.getId()));
  }

  @JsonIgnore
  public boolean isCustomerUser() {
    return !isSystemAdmin() && !isTenantAdmin();
  }
}
