/**
 * Copyright © 2016-2022 The Thingsboard Authors
 *
 * <p>Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License at
 *
 * <p>http://www.apache.org/licenses/LICENSE-2.0
 *
 * <p>Unless required by applicable law or agreed to in writing, software distributed under the
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tofly.base.server.common.data.utils;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;

import java.io.IOException;
import java.util.Arrays;

public class JacksonUtil {

  public static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

  static {
    OBJECT_MAPPER.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
  }

  public static <T> T convertValue(Object fromValue, Class<T> toValueType) {
    try {
      return fromValue != null ? OBJECT_MAPPER.convertValue(fromValue, toValueType) : null;
    } catch (IllegalArgumentException e) {
      throw new IllegalArgumentException(
          "The given object value: " + fromValue + " cannot be converted to " + toValueType, e);
    }
  }

  public static <T> T convertValue(Object fromValue, TypeReference<T> toValueTypeRef) {
    try {
      return fromValue != null ? OBJECT_MAPPER.convertValue(fromValue, toValueTypeRef) : null;
    } catch (IllegalArgumentException e) {
      throw new IllegalArgumentException(
          "The given object value: " + fromValue + " cannot be converted to " + toValueTypeRef, e);
    }
  }

  public static <T> T fromString(String string, Class<T> clazz) {
    try {
      return string != null ? OBJECT_MAPPER.readValue(string, clazz) : null;
    } catch (IOException e) {
      throw new IllegalArgumentException(
          "The given string value: " + string + " cannot be transformed to Json object", e);
    }
  }

  public static <T> T fromString(String string, TypeReference<T> valueTypeRef) {
    try {
      return string != null ? OBJECT_MAPPER.readValue(string, valueTypeRef) : null;
    } catch (IOException e) {
      throw new IllegalArgumentException(
          "The given string value: " + string + " cannot be transformed to Json object", e);
    }
  }

  public static <T> T fromBytes(byte[] bytes, Class<T> clazz) {
    try {
      return bytes != null ? OBJECT_MAPPER.readValue(bytes, clazz) : null;
    } catch (IOException e) {
      throw new IllegalArgumentException(
          "The given string value: "
              + Arrays.toString(bytes)
              + " cannot be transformed to Json object",
          e);
    }
  }

  public static JsonNode fromBytes(byte[] bytes) {
    try {
      return OBJECT_MAPPER.readTree(bytes);
    } catch (IOException e) {
      throw new IllegalArgumentException(
          "The given byte[] value: "
              + Arrays.toString(bytes)
              + " cannot be transformed to Json object",
          e);
    }
  }

  public static String toString(Object value) {
    try {
      return value != null ? OBJECT_MAPPER.writeValueAsString(value) : null;
    } catch (JsonProcessingException e) {
      throw new IllegalArgumentException(
          "The given Json object value: " + value + " cannot be transformed to a String", e);
    }
  }

  public static JsonNode toJsonNode(String value) {
    if (value == null || value.isEmpty()) {
      return null;
    }
    try {
      return OBJECT_MAPPER.readTree(value);
    } catch (IOException e) {
      throw new IllegalArgumentException(e);
    }
  }

  public static ObjectNode newObjectNode() {
    return OBJECT_MAPPER.createObjectNode();
  }

  public static <T> T clone(T value) {
    @SuppressWarnings("unchecked")
    Class<T> valueClass = (Class<T>) value.getClass();
    return fromString(toString(value), valueClass);
  }

  public static <T> JsonNode valueToTree(T value) {
    return OBJECT_MAPPER.valueToTree(value);
  }
}
