package com.tofly.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.GlobalConfigUtils;
import com.baomidou.mybatisplus.core.toolkit.ReflectionKit;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.tofly.base.entity.BaseEntity;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.constant.ModelConstants;
import com.tofly.base.server.common.data.enums.OrderTypeEnum;
import com.tofly.base.server.common.data.utils.ReflectUtils;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.session.SqlSession;
import org.mybatis.spring.SqlSessionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.tofly.base.server.common.data.constant.QueryConstant.*;

public abstract class AbstractBaseService<M extends BaseMapper<T>, T extends BaseEntity>
        implements BaseService<T> {

    @Autowired
    protected M baseMapper;

    @Override
    public List<T> findAllByTenant(String tenantId) {
        QueryWrapper<T> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(tenantId), ModelConstants.TableFields.TENANT_ID, tenantId);
        return baseMapper.selectList(wrapper);
    }

    protected IPage<T> getPage(Map<String, Object> params, String defaultOrderField, boolean isAsc) {
        int curPage = 1;
        int limit = 10;
        if (params.get(PAGE) != null) {
            curPage = Integer.parseInt(params.get(PAGE).toString());
        }

        if (params.get(PAGE_SIZE) != null) {
            limit = Integer.parseInt(params.get(PAGE_SIZE).toString());
        }

        Page<T> page = new Page<>(curPage, limit);
        String orderField = (String) params.get(ORDER_FILED);
        String order = (String) params.get(ORDER_TYPE);
        if (StringUtils.isNotBlank(orderField) && StringUtils.isNotBlank(order)) {
            return OrderTypeEnum.ASC.name().equalsIgnoreCase(order)
                    ? page.addOrder(OrderItem.asc(orderField))
                    : page.addOrder(OrderItem.desc(orderField));
        } else if (StringUtils.isBlank(defaultOrderField)) {
            return page;
        } else {
            if (isAsc) {
                page.addOrder(OrderItem.asc(defaultOrderField));
            } else {
                page.addOrder(OrderItem.desc(defaultOrderField));
            }
            return page;
        }
    }

    protected IPage<T> getCurrentPage(Integer page, Integer pageSize, String orderField, OrderTypeEnum orderType) {
        if (page == null) {
            page = 1;
        }
        if (pageSize == null) {
            pageSize = 10;
        }
        if (StringUtils.isEmpty(orderField)) {
            orderField = FastIotConstants.DefaultOrder.CREATE_TIME;
        }
        if (orderType == null) {
            orderType = OrderTypeEnum.DESC;
        }

        Page<T> pageInform = new Page<>(page, pageSize);

        switch (orderType) {
            case ASC:
                pageInform.addOrder(OrderItem.asc(orderField));
                break;
            default:
                pageInform.addOrder(OrderItem.desc(orderField));
        }

        return pageInform;
    }

    /**
     * 分页信息
     *
     * @param pageNum    第几页
     * @param pageSize   每页数据量
     * @param orderField 排序字段，默认：CREATE_TIME
     * @param orderType  排序方式，默认：DESC
     * @return
     */
    @Override
    public IPage<T> getPage(Integer pageNum, Integer pageSize, String orderField, OrderTypeEnum orderType) {
        int curPage = 1;
        int limit = 10;
        if (pageNum != null) {
            curPage = pageNum;
        }

        if (pageSize != null) {
            limit = pageSize;
        }

        Page<T> page = new Page<>(curPage, limit);

        String realField = FastIotConstants.DefaultOrder.CREATE_TIME;
        if (StringUtils.isNotBlank(orderField)) {
            realField = orderField;
        }

        OrderTypeEnum realType = OrderTypeEnum.DESC;
        if (orderType != null) {
            realType = orderType;
        }

        switch (realType) {
            case ASC:
                page.addOrder(OrderItem.asc(realField));
                break;
            default:
                page.addOrder(OrderItem.desc(realField));
        }

        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteDataByTenantIds(Set<String> tenantIds) {
        if (tenantIds.isEmpty()) {
            return false;
        }
        QueryWrapper<T> wrapper = new QueryWrapper<>();
        wrapper.in(ModelConstants.TableFields.TENANT_ID, tenantIds);
        return baseMapper.delete(wrapper) > FastIotConstants.MagicNumber.ZERO;
    }

    protected <D> TkPageData<D> getPageData(IPage<?> iPage, Class<D> target) {
        List<D> targetList = ReflectUtils.sourceToTarget(iPage.getRecords(), target);
        return new TkPageData<>(targetList, iPage.getTotal());
    }

    /**
     * 批量插入
     *
     * @param entityList
     * @param batchSize
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean insertBatch(Collection<T> entityList, int batchSize) {
        SqlSession batchSqlSession = sqlSessionBatch();
        int i = 0;
        String sqlStatement = sqlStatement(SqlMethod.INSERT_ONE);
        try {
            for (T anEntityList : entityList) {
                batchSqlSession.insert(sqlStatement, anEntityList);
                if (i >= 1 && i % batchSize == 0) {
                    batchSqlSession.flushStatements();
                }
                i++;
            }
            batchSqlSession.flushStatements();
        } finally {
            closeSqlSession(batchSqlSession);
        }
        return true;
    }

    /**
     * 获取SqlStatement
     *
     * @param sqlMethod
     * @return
     */
    protected String sqlStatement(SqlMethod sqlMethod) {
        return SqlHelper.table(currentModelClass()).getSqlStatement(sqlMethod.getMethod());
    }

    /**
     * 批量操作 SqlSession
     */
    protected SqlSession sqlSessionBatch() {
        return SqlHelper.sqlSessionBatch(currentModelClass());
    }

    /**
     * 释放sqlSession
     *
     * @param sqlSession session
     */
    protected void closeSqlSession(SqlSession sqlSession) {
        SqlSessionUtils.closeSqlSession(
                sqlSession, GlobalConfigUtils.currentSessionFactory(currentModelClass()));
    }

    protected Class<T> currentModelClass() {
        return (Class<T>) ReflectionKit.getSuperClassGenericType(getClass(),null,1);
    }
}
