package com.tofly.base.service.menu.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.tofly.base.entity.menu.SysMenuEntity;
import com.tofly.base.mapper.menu.MenuMapper;
import com.tofly.base.mapper.role.RoleMapper;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.core.cache.CacheUtils;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.dto.MenuDTO;
import com.tofly.base.server.common.data.enums.MenuTypeEnum;
import com.tofly.base.server.utils.SetUtil;
import com.tofly.base.service.AbstractBaseService;
import com.tofly.base.service.menu.MenuService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Slf4j
@RequiredArgsConstructor
public class SysMenuServiceImpl extends AbstractBaseService<MenuMapper, SysMenuEntity> implements MenuService {

  private final RoleMapper roleMapper;
  private final CacheUtils cacheUtils;

  @Override
  public List<MenuDTO> getAllMenus(String tenantId) {

    List<SysMenuEntity> menus =
        baseMapper.selectList(new QueryWrapper<SysMenuEntity>().lambda().eq(SysMenuEntity::getTenantId, tenantId));
    Map<String, MenuDTO> menuDTOMap = new LinkedHashMap<>(menus.size());
    menus.forEach(menu -> menuDTOMap.put(menu.getId(), menu.getDTO(MenuDTO.class)));
    return buildMenuDTOTree(menuDTOMap,true,true);
  }

  @Override
  public List<MenuDTO> getMyMenus(
      String tenantId,
      String userId,
      boolean isSysAdmin,
      boolean isTenantAdmin,
      boolean needButton,
      boolean needField) {
    List<MenuDTO> menuDTOs;
    if (isSysAdmin) {
      menuDTOs = baseMapper.selectSysAdminMenu();
    } else if (isTenantAdmin) {
      menuDTOs = baseMapper.selectTenantMenu(tenantId);
    } else {
      menuDTOs = baseMapper.selectMyMenu(userId);
    }
    Map<String, MenuDTO> menuDTOMap = new LinkedHashMap<>(menuDTOs.size());
    menuDTOs.forEach(menuDTO -> menuDTOMap.put(menuDTO.getId(), menuDTO));
    return buildMenuDTOTree(menuDTOMap, needButton,needField);
  }

  private List<MenuDTO> buildMenuDTOTree(Map<String, MenuDTO> menuDTOMap, boolean needButton, boolean needField) {
    List<MenuDTO> result = Lists.newArrayList();
    for (MenuDTO node : menuDTOMap.values()) {
      MenuDTO parent = menuDTOMap.get(node.getParentId());      
      if (parent != null && !(node.getId().equals(parent.getId()))) {
        if (!needButton) {
          if (node.getMeta().get("menuType").asText().equals("2")) {
            continue;
          }
        }
        if(!needButton && needField){
          if (node.getMeta().get("menuType").asText().equals("3")) {
            continue;
          }
        }
        parent.getChildren().add(node);
        continue;
      }
      result.add(node);
    }
    Collections.sort(result);
    result.forEach(node -> Collections.sort(node.getChildren()));
    return result;
  }

  @Override
  @Transactional
  public MenuDTO saveMenu(MenuDTO menuDTO) {
    if (menuDTO.getType() == null) {
      menuDTO.setType(MenuTypeEnum.CUSTOM);
    }
    if (menuDTO.getType().equals(MenuTypeEnum.CUSTOM)
        && StringUtils.isEmpty(menuDTO.getName())) {
      throw new TkDataValidationException("menu name is required when save custom menu");
    }
    int count =
            Math.toIntExact(baseMapper.selectCount(
                    new QueryWrapper<SysMenuEntity>().lambda().eq(SysMenuEntity::getPath, menuDTO.getPath())));
    if (count > 0) {
      throw new TkDataValidationException("menu with path " + menuDTO + " already exist");
    }
    SysMenuEntity menu = menuDTO.getEntity(SysMenuEntity.class);
    int insertCount = baseMapper.insert(menu);
    if (insertCount > 0) {
      cacheUtils.invalidateCacheName(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY);
      menu.copyToDTO(menuDTO);
      return menuDTO;
    }
    return null;
  }

  @Override
  public Optional<MenuDTO> getMenu(String id) {
    return Optional.ofNullable(baseMapper.selectById(id)).map(menu -> menu.getDTO(MenuDTO.class));
  }

  @Override
  @Transactional
  public MenuDTO updateMenu(MenuDTO menuDTO) {
    SysMenuEntity oldMenu = baseMapper.selectById(menuDTO.getId());
    if (oldMenu == null) {
      throw new TkDataValidationException("cannot find menu to update");
    } else {
      if (!Objects.equals(menuDTO.getPermission(), oldMenu.getPermission())) {
        cacheUtils.invalidateCacheName(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY);
      }
      menuDTO.copyToEntity(oldMenu);
      baseMapper.updateById(oldMenu);
      oldMenu.copyToDTO(menuDTO);
    }
    return menuDTO;
  }

  @Override
  @Transactional
  public boolean deleteMenus(String tenantId, String[] menuIds) {
    Set<String> ids = SetUtil.toSet(menuIds);

    // 2. 删除角色对应的menu
    roleMapper.deleteRoleMenuMappingByMenuIds(ids);
    // 3. 删除租户对应menu
    baseMapper.deleteTenantMenuMappingByMenuIds(ids);
    // 4. 删除menu
    baseMapper.deleteBatchIds(ids);
    cacheUtils.invalidateCacheName(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY);
    return true;
  }

  @Override
  @Transactional
  public boolean assignMenuToTenant(String tenantId, String[] menuId) {
    Set<String> ids = SetUtil.toSet(menuId);
    int menuCountInDB =
            Math.toIntExact(baseMapper.selectCount(new QueryWrapper<SysMenuEntity>().lambda().in(SysMenuEntity::getId, ids)));
    if (menuCountInDB != menuId.length) {
      throw new TkDataValidationException("please ensure all menu id are valid");
    }
    Set<String> existMenus = baseMapper.selectTenantMenuIds(tenantId);
    Set<String> toDel = Sets.difference(existMenus, ids);
    Set<String> toAdd = Sets.difference(ids, existMenus);
    if (!toDel.isEmpty()) {
      baseMapper.removeMenuFromTenant(tenantId, toDel);
    }
    if (!toAdd.isEmpty()) {
      baseMapper.addMenuToTenant(tenantId, toAdd);
    }
    if (!toDel.isEmpty() && !toAdd.isEmpty()) {
      cacheUtils.invalidate(
          FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY,
          FastIotConstants.CacheConfigKey.USER_PERMISSION_PREFIX + tenantId);
    }
    return true;
  }

  @Override
  @Transactional
  public boolean assignMenuToRole(String[] menuId, String roleId) {
    Set<String> ids = SetUtil.toSet(menuId);
//    if (!ids.isEmpty()) {
//    	int menuCountInDB =
//            Math.toIntExact(baseMapper.selectCount(new QueryWrapper<SysMenuEntity>().lambda().in(SysMenuEntity::getId, ids)));
//    	if (menuCountInDB != menuId.length) {
//    		throw new TkDataValidationException("please ensure all menu id are valid");
//    	}
//    }
    Set<String> existMenus = baseMapper.selectRoleMenuIds(roleId);
    Set<String> toDel = Sets.difference(existMenus, ids);
    Set<String> toAdd = Sets.difference(ids, existMenus);
    
    if (!toDel.isEmpty()) {
      baseMapper.removeMenuFromRole(roleId, toDel);
    }
    if (!toAdd.isEmpty()) {
      baseMapper.addMenuToRole(roleId, toAdd);
    }
    if (!toAdd.isEmpty() && !toDel.isEmpty()) {
      cacheUtils.invalidateCacheName(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY);
    }
    return true;
  }

  @Override
  public Set<String> getMenuIdsByRoleId(String roleId) {
    return roleMapper.getMenuIdsByRoleId(roleId);
  }

  @Override
  public Map<String, String> findMenuMap(String[] ids) {
    Map<String, String> map = new LinkedHashMap<>();
    if (ids == null || ids.length == 0) return map;
    List<SysMenuEntity> list = baseMapper.selectList(
            new QueryWrapper<SysMenuEntity>().lambda().in(SysMenuEntity::getId, ids));
    if (list != null) {
      list.forEach(e -> {
        map.put(e.getId(), e.getName());
      });
    }
    return map;
  }

  @Override
  public List<MenuDTO> getButtons(String userId) {
	  List<MenuDTO> menuDTOs = baseMapper.selectMyMenu(userId);	  
	  List<MenuDTO> returnList =new ArrayList<MenuDTO>();  
	  for(MenuDTO m : menuDTOs) {
		  if (m.getMeta().get("menuType").asText().equals("2")) {
			   returnList.add(m);
	      }
	  }  
	  return returnList;   
  }
}