package com.tofly.base.service.miniofile;

import com.tofly.base.server.common.data.core.exception.FileStorageException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.FileUploadResponse;
import io.minio.*;
import io.minio.errors.*;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidKeyException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.Objects;


@Service
@Slf4j
@ConditionalOnExpression("'${file.storage.type}'=='minio'")
public class MinioFileStorageService implements FileStorageService {

    private final MinioFileStorageProperties fileStorageProperties;
    private final MinioClient minioClient;

    @Autowired
    public MinioFileStorageService(MinioFileStorageProperties fileStorageProperties)
            throws KeyManagementException {
        this.fileStorageProperties = fileStorageProperties;
        // 创建客户端对象
        this.minioClient =
                MinioClient.builder()
                        .endpoint(fileStorageProperties.getMinioUrl())
                        .credentials(fileStorageProperties.getMinioName(), fileStorageProperties.getMinioPass())
                        .httpClient(getUnsafeOkHttpClient())
                        .build();
        // 创建储存桶
        checkBucket();
    }

    public static OkHttpClient getUnsafeOkHttpClient() throws KeyManagementException {
        try {
            final TrustManager[] trustAllCerts =
                    new TrustManager[]{
                            new X509TrustManager() {
                                @Override
                                public void checkClientTrusted(X509Certificate[] x509Certificates, String s) {
                                }

                                @Override
                                public void checkServerTrusted(X509Certificate[] x509Certificates, String s) {
                                }

                                @Override
                                public X509Certificate[] getAcceptedIssuers() {
                                    return new X509Certificate[]{};
                                }
                            }
                    };

            final SSLContext sslContext = SSLContext.getInstance("SSL");
            sslContext.init(null, trustAllCerts, new SecureRandom());
            final SSLSocketFactory sslSocketFactory = sslContext.getSocketFactory();
            OkHttpClient.Builder builder = new OkHttpClient.Builder();
            builder.sslSocketFactory(sslSocketFactory, (X509TrustManager) trustAllCerts[0]);

            builder.hostnameVerifier((s, sslSession) -> true);
            return builder.build();

        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return null;
    }

    // 储存文件
    public String storeFile(MultipartFile file, String Path) {
        // Normalize file name
        String fileName =  StringUtils.cleanPath(Objects.requireNonNull(file.getOriginalFilename()));
        // random fileName if needed
        // 储存
        try {
            return storeFileByInputStream(Path, fileName, file.getContentType(), file.getInputStream());
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
        }
    }

    @Override
    public FileUploadResponse upload(MultipartFile file, String Path) {
        String fileName = this.storeFile(file, Path);
        String staticPath = getPath(fileName);
        String realDownloadPath = null;
        try {
            // 获取request
            HttpServletRequest request =
                    ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            // 生成下载接口的地址
            realDownloadPath =
                    request.getRequestURL().toString().replace("upload", "downloadFile/" + fileName);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("Could not upload file " + fileName + ". Please try again!");
        }
        return new FileUploadResponse(
                fileName, realDownloadPath, file.getContentType(), file.getSize(), staticPath);
    }

    @Override
    public ResponseEntity<?> download(
            String fileName, HttpServletRequest request, HttpServletResponse response) {
        // get object given the bucket and object name
        try (
                // 获取文件流
                InputStream inputStream =
                        minioClient.getObject(
                                GetObjectArgs.builder()
                                        .bucket(fileStorageProperties.getBucketName())
                                        .object(fileName)
                                        .build());
                // 获取输出流
                OutputStream outputStream = response.getOutputStream()) {
            // 清空
            response.reset();
            // 写入流
            byte[] buffer = inputStream.readAllBytes();
            outputStream.write(buffer);
            outputStream.flush();
            // 配置header
            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + fileName);
            response.addHeader(HttpHeaders.CONTENT_LENGTH, String.valueOf(buffer.length));
            return ResponseEntity.status(HttpStatus.OK).body("download...");
        } catch (Exception e) {
            // 找不到文件时会抛出异常
            String err = fileName + "file not found";
            log.info(err);
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(err);
        }
    }

    @Override
    public String uploadFile(String Path, String fileName, String contentType, InputStream inputStream)
            throws IOException {
        try {
            return getPath(storeFileByInputStream(Path, fileName, contentType, inputStream));
        } catch (Exception e) {
            throw new IOException(e.getMessage());
        }
    }

    @Override
    public void removeFile(String PathfileName) throws IOException {
        try {
            storeFileByemove(PathfileName);
        } catch (Exception e) {
            throw new IOException(e.getMessage());
        }
    }


    public void storeFileByemove(String PathfileName)
            throws ServerException, InsufficientDataException, ErrorResponseException, IOException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {
        RemoveObjectArgs args = RemoveObjectArgs.builder()
                .bucket(fileStorageProperties.getBucketName())
                .object(PathfileName)
                .build();
        minioClient.removeObject(args);
    }


    // 检查储存桶情况
    private void checkBucket() {
        try {
            // 桶是否存在
            boolean exists =
                    minioClient.bucketExists(
                            BucketExistsArgs.builder().bucket(fileStorageProperties.getBucketName()).build());
            if (!exists) {
                // 不存在创建
                minioClient.makeBucket(
                        MakeBucketArgs.builder().bucket(fileStorageProperties.getBucketName()).build());
                // 设置储存桶策略
                String str =
                        "{\"Statement\": [{\"Action\": [\"s3:GetBucketLocation\",\"s3:ListBucket\"],\"Effect\": \"Allow\",\"Principal\": \"*\",\"Resource\": \"arn:aws:s3:::%\"},{\"Action\": \"s3:GetObject\",\"Effect\": \"Allow\",\"Principal\": \"*\",\"Resource\": \"arn:aws:s3:::%/*\"}],\"Version\": \"2012-10-17\"}";
                String replace = str.replace("%", fileStorageProperties.getBucketName());
                minioClient.setBucketPolicy(
                        SetBucketPolicyArgs.builder()
                                .bucket(fileStorageProperties.getBucketName())
                                .config(replace)
                                .build());
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * @param fileName 文件名
     * @return 文件访问地址
     */
    @NotNull
    private String getPath(String fileName) {
        return fileStorageProperties
                .getMinioUrl()
                .concat("/")
                .concat(fileStorageProperties.getBucketName())
                .concat("/")
                .concat(fileName);
    }

    private String storeFileByInputStream(String Path,
                                          String fileName, String contentType, InputStream inputStream)
            throws ServerException, InsufficientDataException, ErrorResponseException, IOException,
            NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException,
            XmlParserException, InternalException {
        if (fileStorageProperties.isRandomFileName()) {
            if (!fileName.contains(".")) {
                fileName = RandomStringUtils.randomAlphabetic(15);
            } else {
                fileName =
                        RandomStringUtils.randomAlphabetic(15) + fileName.substring(fileName.lastIndexOf("."));
            }
        }
        // Check if the file's name contains invalid characters
        if (fileName.contains("..")) {
            throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
        }
        fileName = Path + fileName;
        checkBucket();
        PutObjectArgs build =
                PutObjectArgs.builder()
                        .bucket(fileStorageProperties.getBucketName())
                        .object(fileName)
                        .contentType(contentType)
                        .stream(inputStream, inputStream.available(), -1)
                        .build();
        minioClient.putObject(build);
        return fileName;
    }
}
