package com.tofly.base.service.role.impl;

import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Sets;
import com.tofly.base.entity.role.SysRoleEntity;
import com.tofly.base.mapper.menu.MenuMapper;
import com.tofly.base.mapper.role.RoleMapper;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.core.cache.CacheUtils;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.dto.RoleDTO;
import com.tofly.base.server.common.data.dto.request.RoleReqDTO;
import com.tofly.base.server.common.data.enums.RoleEnum;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.server.utils.SetUtil;
import com.tofly.base.service.AbstractBaseService;
import com.tofly.base.service.menu.MenuService;
import com.tofly.base.service.role.RoleService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Slf4j
@RequiredArgsConstructor
public class SysRoleServiceImpl extends AbstractBaseService<RoleMapper, SysRoleEntity> implements RoleService {

    private final MenuMapper menuMapper;
    private final CacheUtils cacheUtils;
    private final MenuService menuService;
    //private final UserRoleMapper userRoleMapper;

    @Override
    public TkPageData<RoleDTO> page(String tenantId, Map<String, Object> queryMap) {
        IPage<SysRoleEntity> roleIPage = baseMapper.selectPage(getPage(queryMap, "create_time", false),
                new QueryWrapper<SysRoleEntity>().lambda()
                        .eq(queryMap.get("status") != null, SysRoleEntity::isEnabled, queryMap.get("status"))
                        .eq(queryMap.get("roleType") != null, SysRoleEntity::getRoleType, queryMap.get("roleType"))
                        .ne(queryMap.get("roleType") == null, SysRoleEntity::getRoleType, RoleEnum.TENANT_ADMIN.name())
                        .eq(tenantId != null, SysRoleEntity::getTenantId, tenantId)
                        .like(queryMap.get("roleName") != null, SysRoleEntity::getName, String.valueOf(queryMap.get("roleName"))));
        return getPageData(roleIPage, RoleDTO.class);
    }

    @Override
    @Transactional
    public boolean deleteRole(String[] roleIds, String tenantId) {
        Set<String> ids = SetUtil.toSet(roleIds);
        int notTenantMenuCount = Math.toIntExact(baseMapper.selectCount(new QueryWrapper<SysRoleEntity>()
                .lambda()
                .ne(SysRoleEntity::getTenantId, tenantId)
                .in(SysRoleEntity::getId, ids)));
        if (notTenantMenuCount > 0) {
            throw new AccessDeniedException("只能删除自己创建的角色");
        }
        // 判断该角色下面是否有用户
//        Set<String> userIds = baseMapper.checkRoleUserMappingByRoleIds(ids);
//        if (null != userIds && userIds.size() > 0) {
//            throw new TkDataValidationException(ErrorMessage.ROLE_IN_USE.getMessage());
//        }
        // delete sys_user_role mapping
        baseMapper.deleteRoleUserMappingByRoleIds(ids);
        // delete sys_role_menu mapping
        baseMapper.deleteRoleMenuMappingByRoleIds(ids);
        // delete role
        baseMapper.deleteBatchIds(ids);
        // FIXME 频繁删除role造成缓存效果不大
        cacheUtils.invalidateCacheName(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY);
        return true;
    }

    @Override
    public Set<String> getPermissions(boolean isSysadmin, boolean isTenantAdmin, String tenantId, String useerId) {
        Set<String> permissions = Sets.newHashSet();
        Set<String> allPermission;
        String cacheKey;
        boolean cachePresent;
        if (isSysadmin) {
            cacheKey = FastIotConstants.CacheConfigKey.USER_PERMISSION_PREFIX + RoleEnum.SYS_ADMIN.name();
            Optional<Set<String>> optionalPermission = cacheUtils.get(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY, cacheKey);
            cachePresent = optionalPermission.isPresent();
            allPermission = optionalPermission.orElseGet(menuMapper::getAllPermission);
        } else if (isTenantAdmin) {
            cacheKey = FastIotConstants.CacheConfigKey.USER_PERMISSION_PREFIX + tenantId;
            Optional<Set<String>> optionalPermission = cacheUtils.get(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY, cacheKey);
            cachePresent = optionalPermission.isPresent();
            allPermission = optionalPermission.orElseGet(() -> menuMapper.getAllPermissionsByTenantId(tenantId));
        } else {
            cacheKey = FastIotConstants.CacheConfigKey.USER_PERMISSION_PREFIX + useerId;
            Optional<Set<String>> optionalPermission = cacheUtils.get(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY, cacheKey);
            cachePresent = optionalPermission.isPresent();
            allPermission = optionalPermission.orElseGet(() -> menuMapper.getAllPermissionsByUserId(useerId));
        }
        if (cachePresent) {
            return allPermission;
        } else {
            allPermission.forEach(permission -> permissions.addAll(Sets.newHashSet(permission.split(","))));
            cacheUtils.put(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY, cacheKey, permissions);
        }
        return permissions;
    }

    @Override
    @Transactional
    public void updateRoleStatus(String roleId, int status, boolean isSysadmin, String tenantId) {
        Optional.ofNullable(baseMapper.selectOne(new QueryWrapper<SysRoleEntity>().lambda().eq(SysRoleEntity::getId, roleId).eq(!isSysadmin, SysRoleEntity::getTenantId, tenantId))).ifPresent(role -> {
            role.setEnabled(status == 1);
            baseMapper.updateById(role);
        });
    }

    @Override
    @Transactional
    public RoleDTO saveOrUpdateRoleInfoWithMenu(RoleReqDTO roleReqDTO, boolean isSysadmin, boolean isPlatformAdmin, String tenantId) throws TkDataValidationException {
        SysRoleEntity role;
        // 默认普通管理员角色,即租户管理员添加的角色
        String roleType = RoleEnum.CUSTOMER_USER.name();
        if (null != roleReqDTO.getRoleType()) {
            roleType = roleReqDTO.getRoleType().name();
        } else {
            if (isSysadmin || isPlatformAdmin) {
                roleType = RoleEnum.PLATFORM_ADMIN.name();
            }
        }
        boolean update = StringUtils.isNotBlank(roleReqDTO.getId());
        if (update) {
            // do update
            role = baseMapper.selectOne(new QueryWrapper<SysRoleEntity>().lambda().eq(SysRoleEntity::getId, roleReqDTO.getId()).eq(!isSysadmin, SysRoleEntity::getTenantId, tenantId));
            if (role == null) {
                return null;
            } else {
                role.setRemark(roleReqDTO.getRemark());
                role.setName(roleReqDTO.getName());
                role.setEnabled(roleReqDTO.getStatus() == 1);
                baseMapper.updateById(role);
            }
        } else {
        	role = new SysRoleEntity();
        	
        	SysRoleEntity existRole = baseMapper.selectOne(new QueryWrapper<SysRoleEntity>().lambda()
        			  .eq(SysRoleEntity::getName, roleReqDTO.getName()));
        	
        	if(null==existRole) {
                role.setRoleType(roleType);
                role.setCode(RandomStringUtils.randomAlphabetic(10));
                role.setRemark(roleReqDTO.getRemark());
                role.setName(roleReqDTO.getName());
                role.setEnabled(roleReqDTO.getStatus() == 1);
                role.setTenantId(tenantId);
                role.setCreateTime(new Date());
                baseMapper.insert(role);
        	}else {
        		throw new TkDataValidationException("已存在相同角色名称");
        	}
        }
//        Set<String> oldMenus = menuMapper.selectRoleMenuIds(role.getId());
        // do update or save menu associate with this roleId
        menuService.assignMenuToRole(roleReqDTO.getMenu().toArray(new String[roleReqDTO.getMenu().size()]), role.getId());
        // 如果是租户管理员角色并且是更新，则需要更新租户菜单表
//        if (role.getRoleType().equals(RoleEnum.TENANT_ADMIN.name()) && update) {
//            List<String> newMenus = roleReqDTO.getMenu();
//            Set<String> removeMenus = new HashSet<>();
//            for (String menu : oldMenus) {
//                if (newMenus.contains(menu)) {
//                    newMenus.remove(menu);
//                } else {
//                    removeMenus.add(menu);
//                }
//            }
            // 先删除以前的租户菜单，再更新新的租户菜单
            // 1、查询这个角色有几个租户用户
            // 2、删除并更新对应租户的信息
            // 3、超级管理员删除租户角色菜单时，同步删除租户内客户角色的权限，租户ID+菜单ID
//            List<SysTenantRoleEntity> tenantRoles =
//                    tenantRoleMapper.selectList(
//                            new QueryWrapper<SysTenantRoleEntity>().lambda().eq(SysTenantRoleEntity::getRoleId, role.getId()));
//            tenantRoles.forEach(
//                    tenantRole -> {
//                        String updateTenantId = tenantRole.getTenantId();
//
//                        if(!removeMenus.isEmpty()){
//                            //刷新租户的菜单
//                            tenantMenuMapper.delete(
//                                    new QueryWrapper<SysTenantMenuEntity>()
//                                            .lambda()
//                                            .eq(SysTenantMenuEntity::getTenantId, updateTenantId)
//                                            .in(SysTenantMenuEntity::getMenuId,removeMenus));
//                            //刷新客户的菜单
//                            List<SysRoleEntity> customerRoles = baseMapper.selectList(
//                                    new QueryWrapper<SysRoleEntity>()
//                                            .lambda()
//                                            .eq(SysRoleEntity::getTenantId, updateTenantId)
//                                            .eq(SysRoleEntity::getRoleType,RoleEnum.CUSTOMER_USER.name()));
//                            customerRoles.forEach(cr -> menuMapper.removeMenuFromRole(cr.getId(),removeMenus));
//                        }
//                        newMenus.forEach(
//                                menu -> {
//                                    SysTenantMenuEntity tenantMenu = new SysTenantMenuEntity();
//                                    tenantMenu.setMenuId(menu);
//                                    tenantMenu.setTenantId(updateTenantId);
//                                    tenantMenuMapper.insert(tenantMenu);
//                                });
//
//                    });
//        }
        cacheUtils.invalidateCacheName(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY);
        return role.getDTO(RoleDTO.class);
    }

    @Override
    public List<RoleDTO> findRoleInfo(boolean isTenantAdmin, String tenantId, String userId, RoleDTO roleDTO) {
        if (StringUtils.isEmpty(roleDTO.getTenantId())) {
            roleDTO.setTenantId(tenantId);
        }
        if (isTenantAdmin) {
            // 租户管理员既要查询自己拥有的角色，也要查询自己创建的角色
         //   List<String> roleIds = userRoleMapper.selectList(new QueryWrapper<SysUserRoleEntity>().lambda().eq(SysUserRoleEntity::getUserId, userId)).stream().map(SysUserRoleEntity::getRoleId).collect(Collectors.toList());
         //   roleDTO.setRoleIds(roleIds.toArray(new String[roleIds.size()]));
        }
        return baseMapper.findRoleInfo(roleDTO);
    }

    @Override
    public Map<String, String> findRoleMap(String[] ids) {
        Map<String, String> map = new LinkedHashMap<>();
        if (ids == null || ids.length == 0) return map;
        List<SysRoleEntity> list = baseMapper.selectList(new QueryWrapper<SysRoleEntity>().lambda().in(SysRoleEntity::getId, ids));
        if (list != null) {
            list.forEach(e -> {
                map.put(e.getId(), e.getName());
            });
        }
        return map;
    }

    @Override
    public Boolean getHasRoleByUserId(String userID, String roleName) {
        SysRoleEntity sysRoleEntity = baseMapper.selectOne(Wrappers.lambdaQuery(SysRoleEntity.class).eq(SysRoleEntity::getName, roleName));
        Assert.notNull(sysRoleEntity, "不存在名称为{}的角色",roleName);
        return baseMapper.getHasRoleByUserId(userID, roleName);
    }
}
