package com.tofly.base.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tofly.base.entity.organization.TkOrganizationEntity;
import com.tofly.base.entity.organization.TkUserOrganizationMappingEntity;
import com.tofly.base.mapper.organization.OrganizationMapper;
import com.tofly.base.mapper.organization.UserOrganizationMappingMapper;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.BaseDTO;
import com.tofly.base.server.common.data.dto.OrganizationDTO;
import com.tofly.base.service.user.UserOrganizationMappingService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: 徐浩然
 * @desc: 组织用户关联表
 * @date: 2021/12/3-10:18
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SysUserOrganizationMappingServiceImpl implements UserOrganizationMappingService {

    private final UserOrganizationMappingMapper userOrganizationMappingMapper;
    private final OrganizationMapper organizationMapper;

    @Override
    public List<String> getUserIdByOrganizationIds(String tenantId, String... ids) {

        List<String> collect = Arrays.asList(ids);

        if (null == collect || collect.size() == 0) {
            return new ArrayList<>();
        } else {
            Set<String> clearList = new HashSet<>();
            clearList.addAll(collect);
            return userOrganizationMappingMapper
                    .selectList(
                            new LambdaQueryWrapper<TkUserOrganizationMappingEntity>()
                                    .in(TkUserOrganizationMappingEntity::getOrganizationId, clearList))
                    .stream()
                    .map(TkUserOrganizationMappingEntity::getUserId)
                    .distinct()
                    .collect(Collectors.toList());
        }


    }

    @Override
    @Transactional
    public void addOrUpdateUserOrganizationMapping(
            String userId, List<String> organizationIds, boolean isUpdate) {
        if (StringUtils.isEmpty(userId) || null == organizationIds || organizationIds.size() < 1) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        if (isUpdate) {
            // 先删除,再新增
            userOrganizationMappingMapper.delete(
                    new LambdaQueryWrapper<TkUserOrganizationMappingEntity>()
                            .eq(TkUserOrganizationMappingEntity::getUserId, userId));
        }
        batchInsert(userId, organizationIds);
    }

    @Override
    public List<String> getOrganizationIdsByUserId(String userId) {
        if (StringUtils.isEmpty(userId)) {
            throw new TkDataValidationException(ErrorMessage.INTERNAL_ERROR.getMessage());
        }
        return userOrganizationMappingMapper
                .selectList(
                        new QueryWrapper<TkUserOrganizationMappingEntity>()
                                .lambda()
                                .eq(TkUserOrganizationMappingEntity::getUserId, userId))
                .stream()
                .map(TkUserOrganizationMappingEntity::getOrganizationId)
                .collect(Collectors.toList());
    }

    @Override
    public List<String> getOrganizationNamesByUserId(String id) {
        List<String> orgIds = getOrganizationIdsByUserId(id);
        if (CollectionUtil.isNotEmpty(orgIds)) {
            List<TkOrganizationEntity> orgs = organizationMapper.selectBatchIds(orgIds);
            return orgs.stream().map(TkOrganizationEntity::getName).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }


    private void batchInsert(String userId, List<String> organizationIds) {
        organizationIds.forEach(
                organizationId -> {
                    TkUserOrganizationMappingEntity userOrganizationMapping = new TkUserOrganizationMappingEntity();
                    userOrganizationMapping.setUserId(userId);
                    userOrganizationMapping.setOrganizationId(organizationId);
                    userOrganizationMappingMapper.insert(userOrganizationMapping);
                });
    }

    @Override
    public List<String> getEnableOrganizationIds(
            boolean isPtTenantAdmin, String tenantId, String userId, String organizationId) {
        List<String> organizationIds = null;
        if (StringUtils.isNotEmpty(organizationId)) {
            List<OrganizationDTO> organizationList =
                    organizationMapper.findOrganizationTreeList(
                            tenantId, new HashSet<>(List.of(organizationId)));
            if (organizationList.size() == FastIotConstants.MagicNumber.ZERO) {
                throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
            }
            organizationIds =
                    organizationList.stream().map(BaseDTO::getId).distinct().collect(Collectors.toList());
        } else {
            if (!isPtTenantAdmin) {
                organizationIds = getOrganizationIdsByUserId(userId);
            }
        }
        return organizationIds;
    }

    @Override
    public List<String> compareOrganizationIdsByCustomerId(
            List<String> organizationIds, String userId) {
        int zero = FastIotConstants.MagicNumber.ZERO;
        List<String> list = getOrganizationIdsByUserId(userId);
        if (organizationIds != null && organizationIds.size() > zero && list.size() > zero) {
            List<String> result = new ArrayList<>();
            for (String id : organizationIds) {
                if (list.contains(id)) {
                    result.add(id);
                }
            }
            if (result.size() > zero) {
                return result;
            }
        }
        return list;
    }
}
