package com.tofly.base.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Sets;
import com.tofly.base.ding.handle.DingUserHandle;
import com.tofly.base.entity.organization.TkOrganizationEntity;
import com.tofly.base.entity.organization.TkUserOrganizationMappingEntity;
import com.tofly.base.entity.role.SysRoleEntity;
import com.tofly.base.entity.user.*;
import com.tofly.base.mapper.organization.OrganizationMapper;
import com.tofly.base.mapper.organization.UserOrganizationMappingMapper;
import com.tofly.base.mapper.role.RoleMapper;
import com.tofly.base.mapper.user.*;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.constant.ModelConstants;
import com.tofly.base.server.common.data.core.cache.CacheUtils;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.PlainRolesDTO;
import com.tofly.base.server.common.data.dto.UserDTO;
import com.tofly.base.server.common.data.dto.UserDetailsDTO;
import com.tofly.base.server.common.data.dto.UserInfoDTO;
import com.tofly.base.server.common.data.dto.request.AccountReqDTO;
import com.tofly.base.server.common.data.dto.request.CodeTTL;
import com.tofly.base.server.common.data.dto.request.RoleOrOrganizationReqDTO;
import com.tofly.base.server.common.data.dto.request.SendResetPasswordEmailMsg;
import com.tofly.base.server.common.data.enums.MessageTypeEnum;
import com.tofly.base.server.common.data.enums.MsgTemplatePurposeEnum;
import com.tofly.base.server.common.data.enums.UserStatusEnum;
import com.tofly.base.server.common.data.id.EntityId;
import com.tofly.base.server.common.data.id.UserId;
import com.tofly.base.server.common.data.query.TsValue;
import com.tofly.base.server.common.data.utils.ReflectUtils;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.server.utils.AccountProperties;
import com.tofly.base.service.AbstractBaseService;
import com.tofly.base.service.organization.TkOrganizationService;
import com.tofly.base.service.user.TkUserService;
import com.tofly.base.service.user.UserOrganizationMappingService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.scheduling.annotation.Async;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

import static com.tofly.base.server.common.data.constant.FastIotConstants.CHINA_MOBILE_PATTERN;
import static com.tofly.base.server.common.data.constant.FastIotConstants.EMAIL_PATTERN;
import static com.tofly.base.server.common.data.constant.ModelConstants.TablePropertyMapping.*;


@Service
@Slf4j
@RequiredArgsConstructor
public class SysUserServiceImpl extends AbstractBaseService<UserMapper, SysUserEntity> implements TkUserService {

    public static final String ACTIVATE_URL_PATTERN = "%s/api/noauth/activate?activateToken=%s";

    private final AdminSettingMapper adminSettingMapper;

    private final ObjectMapper mapper;
    private final RoleMapper roleMapper;
    private final OrganizationMapper organizationMapper;
    private final TkOrganizationService organizationService;
    //  private final UserService tbUserService;
    private final ApplicationEventPublisher eventPublisher;
    private final PasswordEncoder passwordEncoder;
    private final TenantMapper tenantMapper;
    private final UserRoleMapper userRoleMapper;
    private final TenantRoleMapper tenantRoleMapper;

    private final UserOrganizationMappingMapper userOrganizationMappingMapper;


    private final CacheUtils cacheUtils;


//  private final TkSmsService tkSmsService;
//  private final TkMessageTemplateService messageTemplateService;

    private final UserOrganizationMappingService userOrganizationMappingService;


    private final AccountProperties accountProperties;


    @Override
    public List<UserDetailsDTO> findUserDetailsByUsername(String username, String tenantId) {
        // 多个租户可能存在多个username相同的情况
        return baseMapper.findUserDetailsByUserName(username, tenantId);
    }

    @Override
    public UserDTO findUserInfoById(String id) {
        if (StringUtils.isEmpty(id)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        SysUserEntity user = baseMapper.selectById(id);
        return null != user ? user.getDTO(UserDTO.class) : null;
    }

    @Override
    public String getActivationLink(String userId, boolean isPtSysadmin, String tenantId) {
        SysUserEntity user = baseMapper.selectById(userId);
        if (!isPtSysadmin && tenantId.equals(user.getTenantId())) {
            throw new AccessDeniedException(
                    "you don't have permission to get activation link for this user");
        }
        if (StringUtils.isEmpty(user.getActivateToken())) {
            throw new TkDataValidationException("user already activated");
        }
        try {
            List<SysAdminSettingEntity> generalSetting =
                    adminSettingMapper.selectList(
                            new QueryWrapper<SysAdminSettingEntity>()
                                    .lambda()
                                    .eq(
                                            SysAdminSettingEntity::getKey,
                                            ModelConstants.AdminSettingConfigKey.GENERAL_SETTING));
            SysAdminSettingEntity adminSetting = generalSetting.get(0);
            JsonNode configNode = mapper.readTree(adminSetting.getConfigJson());
            String baseUrl = configNode.get(FastIotConstants.ConfigJSONKey.BASE_URL).asText();
            return String.format(ACTIVATE_URL_PATTERN, baseUrl, user.getActivateToken());
        } catch (Exception e) {
            throw new RuntimeException("error occurs");
        }
    }

    @Override
    public UserInfoDTO me(String userId, String tenantId, Set<String> currentRoles) {
        SysUserEntity user = baseMapper.selectById(userId);
        UserInfoDTO userInfoDTO = new UserInfoDTO();
        if (!accountProperties.getReset() || !StringUtils.isEmpty(user.getActivateToken())) {
            userInfoDTO.setNeedSetPwd(false);
        }
        AtomicReference<Date> expireTime = new AtomicReference<>(user.getAccountExpireTime());
        if (expireTime.get() == null) {
            SysTenantEntity tenant =
                    tenantMapper.selectOne(
                            new QueryWrapper<SysTenantEntity>()
                                    .lambda()
                                    .eq(SysTenantEntity::getTenantId, tenantId));
            Optional.ofNullable(tenant)
                    .ifPresent(
                            i -> {
                                expireTime.set(i.getTenantExpireTime());
                            });
        }
        userInfoDTO.setAccountExpireTime(expireTime.get());
        userInfoDTO.setUserId(userId);
        userInfoDTO.setUsername(user.getUsername());
        userInfoDTO.setCreateTime(user.getCreateTime());
        userInfoDTO.setRealName(user.getRealName());
        userInfoDTO.setAvatar(user.getAvatar());
        userInfoDTO.setPhoneNumber(user.getPhoneNumber());
        userInfoDTO.setEmail(user.getEmail());
        userInfoDTO.setTenantId(tenantId);
        userInfoDTO.setRoles(currentRoles);
        Set<SysRoleEntity> roles = roleMapper.selectRoleByUserId(userId);
        Set<PlainRolesDTO> plainRolesDTOSet = Sets.newHashSet();
        roles.forEach(
                role -> {
                    PlainRolesDTO plainRolesDTO = new PlainRolesDTO();
                    plainRolesDTO.setRoleId(role.getId());
                    plainRolesDTO.setRoleName(role.getName());
                    plainRolesDTOSet.add(plainRolesDTO);
                });
        userInfoDTO.setPlainRoles(plainRolesDTOSet);
        return userInfoDTO;
    }

    @Override
    public TkPageData<UserDTO> tenantPage(Map<String, Object> queryMap, String tenantId) {
        IPage<SysUserEntity> userIPage = getPage(queryMap, "create_time", false);
        IPage<UserDTO> userPage = baseMapper.getTenantAdminPage(userIPage, tenantId);
        if (null != userPage) {
            userPage
                    .getRecords()
                    .forEach(
                            record -> {
                                fillUserStatus(record);
                                record.setHasPassword(StringUtils.isNotBlank(record.getActivateToken()));
                                record.setPassword(null);
                            });
        }
        return getPageData(userPage, UserDTO.class);
    }

    private void fillUserStatus(UserDTO userDTO) {
        userDTO.setUserStatusEnum(UserStatusEnum.NORMAL);
        if (!userDTO.isEnabled()) {
            userDTO.setUserStatusEnum(UserStatusEnum.DISABLED);
        } else {
            if (userDTO.getAccountExpireTime() != null) {
                if (userDTO.getAccountExpireTime().before(new Date())) {
                    userDTO.setUserStatusEnum(UserStatusEnum.EXPIRED);
                }
            }
        }
    }

    @Override
    @Transactional
    public UserDTO saveTenantAdmin(UserDTO userDTO, boolean isPtSysadmin, String tenantId) {
        if (StringUtils.isNotBlank(userDTO.getId())) {
            return updateUser(userDTO, isPtSysadmin, tenantId);
        }
        SysUserEntity user = new SysUserEntity();
        userDTO.copyToEntity(user, ModelConstants.TablePropertyMapping.ID, ModelConstants.TablePropertyMapping.PASSWORD, ModelConstants.TablePropertyMapping.CREATE_TIME, ModelConstants.TablePropertyMapping.UPDATE_TIME, ModelConstants.TablePropertyMapping.ACTIVATE_TOKEN);
        user.setPassword(passwordEncoder.encode(accountProperties.getDefaultPassword()));
        user.setLevel(FastIotConstants.LevelValue.IS_TENANT_ADMIN);
        List<SysUserEntity> users =
                baseMapper.selectList(
                        new QueryWrapper<SysUserEntity>()
                                .lambda()
                                .eq(SysUserEntity::getUsername, userDTO.getUsername()));
        if (!users.isEmpty()) {
            throw new TkDataValidationException("用户已存在");
        }
        baseMapper.insert(user);
        List<SysTenantRoleEntity> tenantRoleList = validateTenantRole(userDTO.getTenantId());
        for (SysTenantRoleEntity tenantRole : tenantRoleList) {
            roleMapper.saveUserRoleMapping(user.getId(), tenantRole.getRoleId());
        }
        user.copyToDTO(userDTO, ModelConstants.TablePropertyMapping.PASSWORD, ModelConstants.TablePropertyMapping.ACTIVATE_TOKEN);
        return userDTO;
    }

    @Override
    public List<SysTenantRoleEntity> validateTenantRole(String tenantId) {
        List<SysTenantRoleEntity> tenantRoleList =
                tenantRoleMapper.selectList(
                        new QueryWrapper<SysTenantRoleEntity>()
                                .lambda()
                                .eq(SysTenantRoleEntity::getTenantId, tenantId));
        // 保存用户与角色的映射信息
        if (null == tenantRoleList || tenantRoleList.size() == 0) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        return tenantRoleList;
    }

    @Override
    @Transactional
    public void resetPassword(String userId, String tenantId, String password) {
        UserDTO userDTO = findUserInfoById(userId);
        if (null == userDTO) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        userDTO.setPassword(passwordEncoder.encode(password));
        userDTO.setActivateToken(null);
        baseMapper.updateById(userDTO.getEntity(SysUserEntity.class));
    }

    @Override
    public void forgetPassword(String phoneNumber, AccountReqDTO forget) {
        String key =
                MsgTemplatePurposeEnum.FOR_FORGET_PASSWORD.name()
                        + FastIotConstants.DEFAULT_DELIMITER
                        + MessageTypeEnum.PHONE_MESSAGE.name()
                        + FastIotConstants.DEFAULT_DELIMITER
                        + phoneNumber;


        boolean correct =
                cacheUtils
                        .get(FastIotConstants.CacheConfigKey.MOBILE_LOGIN_SMS_CODE, key)
                        .map(
                                o -> {
                                    CodeTTL codeTTL = (CodeTTL) o;
                                    if (System.currentTimeMillis() - codeTTL.getSendTs() < 5 * 60 * 1000) {
                                        return Objects.equals(codeTTL.getCode(), forget.getUserId());
                                    } else {
                                        return false;
                                    }
                                })
                        .orElse(false);
        if (!correct) {
            throw new TkDataValidationException(ErrorMessage.MSG_CODE_NOT_MATCHED.getMessage());
        }
        String pwd = forget.getPassword();
        if (StringUtils.isEmpty(pwd)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }

        SysUserEntity user =
                baseMapper.selectOne(
                        new QueryWrapper<SysUserEntity>()
                                .lambda()
                                .eq(SysUserEntity::getPhoneNumber, phoneNumber));

        if (StringUtils.isNotBlank(user.getTbUser())) {
            UserId userId = new UserId(UUID.fromString(user.getTbUser()));
//      UserCredentials userCredentials =
//          tbUserService.findUserCredentialsByUserId(TenantId.SYS_TENANT_ID, userId);
//
//      userCredentials.setPassword(passwordEncoder.encode(pwd));
//      tbUserService.replaceUserCredentials(
//          TenantId.fromUUID(UUID.fromString(user.getTenantId())), userCredentials);
            //    eventPublisher.publishEvent(new UserAuthDataChangedEvent(userId));
        }

        user.setPassword(pwd);
        changePassword(user);
    }

    @Override
    public List<UserDetailsDTO> getUserByPhoneNumber(String phoneNumber) {
        return baseMapper.findUserDetailsByPhoneNumber(phoneNumber);
    }

    @Override
    public void sendRestPasswordMsg(String tenantId, SendResetPasswordEmailMsg msg) {
        // 通过用户ID查询用户信息
        SysUserEntity user = baseMapper.selectById(msg.getUserId());
        if (null == user) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }

        // 通过模板用途和状态查询模板信息
//    MessageTemplateDTO messageTemplateDTO = new MessageTemplateDTO();
//    String messageType = msg.getMessageTypeEnum().name();
//    messageTemplateDTO.setMessageType(messageType);
//    messageTemplateDTO.setTemplatePurpose(MsgTemplatePurposeEnum.FOR_SET_PASSWORD.name());
//    messageTemplateDTO.setTenantId(tenantId);
//    messageTemplateDTO.setStatus(1);
//    List<MessageTemplateDTO> templateDTOList =
//        messageTemplateService.findMessageTemplate(messageTemplateDTO);
//    if (null == templateDTOList || templateDTOList.size() < 1) {
//      throw new TkDataValidationException(
//          ErrorMessage.TARGET_TEMPLATE_NOT_EXISTS_SELF_NOTICE.getMessage());
//    }
//    if (messageType.equalsIgnoreCase(MessageTypeEnum.PHONE_MESSAGE.name())) {
//      SmsReqDTO smsReqDTO = new SmsReqDTO();
//      smsReqDTO.setPhoneNumbers(user.getPhoneNumber());
//      smsReqDTO.setId(templateDTOList.get(0).getId());
//      LinkedHashMap<String, String> params = new LinkedHashMap<>();
//      params.put("code", accountProperties.getDefaultPassword());
//      smsReqDTO.setParams(params);
//      smsReqDTO.setTemplatePurpose(messageTemplateDTO.getTemplatePurpose());
//      tkSmsService.sendSms(smsReqDTO);
//    }
    }

    @Override
    public Optional<List<UserDTO>> getOrganizationUserByOrganizationId(
            String groupId, String tenantId) {
        TkOrganizationEntity group = organizationMapper.selectById(groupId);
        if (group == null || !group.getTenantId().equals(tenantId)) {
            return Optional.empty();
        }
        List<String> userIds = userOrganizationMappingService.getUserIdByOrganizationIds(group.getId());
        List<SysUserEntity> users = baseMapper.selectBatchIds(userIds);
        return Optional.ofNullable(ReflectUtils.sourceToTarget(users, UserDTO.class));
    }

    @Override
    @Transactional
    public boolean changePassword(SysUserEntity user) {
        // 修改密码
        user.setPassword(passwordEncoder.encode(user.getPassword()));
        user.setActivateToken(RandomStringUtils.randomAlphabetic(10));
        return baseMapper.updateById(user) > 0;
    }

    @Override
    public String findUserCustomerIdById(String id) {
        if (StringUtils.isEmpty(id)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        String customerId = baseMapper.findUserCustomerIdById(id);
        if (null == customerId) {
            throw new TkDataValidationException(ErrorMessage.USER_NOT_EXISTS.getMessage());
        }
        return customerId;
    }

    @Override
    public UserDTO updatePersonalCenter(UserDTO userDTO) {
        SysUserEntity user = baseMapper.selectById(userDTO.getId());
        user.setRealName(userDTO.getRealName());
        user.setEmail(userDTO.getEmail());
        user.setAvatar(userDTO.getAvatar());
        user.setPhoneNumber(userDTO.getPhoneNumber());
        baseMapper.updateById(user);
        user.copyToDTO(userDTO);
        return userDTO;
    }

    @Override
    public List<UserDTO> getMyCustomers(
            String organizationId, String tenantId, boolean isTenantAdmin) {
        if (StringUtils.isEmpty(organizationId)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        if (isTenantAdmin) {
            List<String> userList =
                    userOrganizationMappingService.getUserIdByOrganizationIds(tenantId, organizationId);
            if (null != userList && userList.size() > FastIotConstants.MagicNumber.ZERO) {
                return baseMapper.getMyCustomers(tenantId, EntityId.NULL_UUID.toString(), userList);
            }
        }
        return null;
    }

    @Override
    @Async
    public CompletableFuture<TsValue> findUsersAsyncByTs(
            LocalDateTime startTs, LocalDateTime endTs, long ts) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String startTime = df.format(startTs);
        String endTime = df.format(endTs);
        List<UserDTO> list = baseMapper.findUsersAsyncByTs(startTime, endTime);
        return CompletableFuture.supplyAsync(() -> new TsValue(ts, String.valueOf(list.size())));
    }

    @Override
    public UserDTO checkAccount(String userId, Integer level) {
        SysUserEntity user =
                baseMapper.selectOne(
                        new LambdaQueryWrapper<SysUserEntity>()
                                .eq(SysUserEntity::getId, userId)
                                .eq(SysUserEntity::getLevel, level));
        return null != user ? user.getDTO(UserDTO.class) : null;
    }

    @Override
    public Map<String, String> findUserMap(Set<String> ids) {
        Map<String, String> map = new LinkedHashMap<>();
        if (ids == null || ids.size() == 0) return map;
        List<SysUserEntity> list = baseMapper.selectList(
                new QueryWrapper<SysUserEntity>().lambda().in(SysUserEntity::getId, ids));
        if (list != null) {
            list.forEach(e -> {
                map.put(e.getId(), e.getUsername());
            });
        }
        return map;
    }

    @Override
    public UserDTO getUserByPhoneNumberOrUsername(String principal) {
        SysUserEntity user =

                baseMapper.selectOne(
                        new LambdaQueryWrapper<SysUserEntity>()
                                .eq(SysUserEntity::getUsername, principal));
        if (user == null) {
            user = baseMapper.selectOne(
                    new LambdaQueryWrapper<SysUserEntity>()
                            .eq(SysUserEntity::getPhoneNumber, principal));
        }
        return null != user ? user.getDTO(UserDTO.class) : null;
    }

    @Override
    public Set<String> getUserRole(String userId) {
        Set<String> roleIds =
                userRoleMapper
                        .selectList(
                                new QueryWrapper<SysUserRoleEntity>()
                                        .lambda()
                                        .eq(SysUserRoleEntity::getUserId, userId))
                        .stream()
                        .map(SysUserRoleEntity::getRoleId)
                        .collect(Collectors.toSet());
        if (roleIds.size() == 0) return new LinkedHashSet<>();
        Set<String> enableRoles =
                roleMapper
                        .selectList(
                                new QueryWrapper<SysRoleEntity>()
                                        .lambda()
                                        .eq(SysRoleEntity::isEnabled, true)
                                        .in(SysRoleEntity::getId, roleIds))
                        .stream()
                        .map(m -> m.getRoleType())
                        .collect(Collectors.toSet());
        return enableRoles;
    }


    @Override
    public SysUserEntity validateChangePasswordAccount(AccountReqDTO accountReqDTO) {
        SysUserEntity user = null;
        if (null != accountReqDTO.getUserId()) {
            user = baseMapper.selectById(accountReqDTO.getUserId());
        } else if (null != accountReqDTO.getUserName()) {
            user = baseMapper.selectOne(
                    new LambdaQueryWrapper<SysUserEntity>()
                            .eq(SysUserEntity::getUsername, accountReqDTO.getUserName()));
        }
        checkPassword(accountReqDTO, user);
        return user;
    }

    private void checkPassword(AccountReqDTO accountReqDTO, SysUserEntity user) {
        if (null == user
                || StringUtils.isEmpty(accountReqDTO.getPassword())
                || StringUtils.isEmpty(accountReqDTO.getResetPassword())) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        // 判断用户密码是否正确
        boolean isMatch = passwordEncoder.matches(accountReqDTO.getPassword(), user.getPassword());
        if (!isMatch) {
            throw new TkDataValidationException(ErrorMessage.PASSWORD_INCORRECT.getMessage());
        }
        user.setPassword(accountReqDTO.getResetPassword());
    }

    /**
     * 先删除用户与角色的关系，再添加新的关系
     *
     * @param userId  用户ID
     * @param roleIds 角色ID
     */
    private void deleteAndAddUserRole(String userId, List<String> roleIds) {
        if (StringUtils.isEmpty(userId) || null == roleIds || roleIds.size() < 1) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        // 先删除用户与角色的关系,再添加用户与角色的关系
        List<String> userIds = new ArrayList<>();
        userIds.add(userId);
        roleMapper.deleteRoleUserMappingByUserIds(userIds);
        roleIds.forEach(
                role -> {
                    roleMapper.saveUserRoleMapping(userId, role);
                });
    }


    @Override
    public TkPageData<UserDTO> page(Map<String, Object> queryMap, boolean isPtSysadmin, boolean isTenantAdmin) {
        String tenantId = (String) queryMap.get("tenantId");
        IPage<UserDTO> userPage = null;
        IPage<SysUserEntity> userIPage = getPage(queryMap, "create_time", false);
        String organizationId = queryMap.get("organizationId") != null ? (String) queryMap.get("organizationId") : null;
        List<String> userIds = null;
        if (null != organizationId) {
            userIds = userOrganizationMappingService.getUserIdByOrganizationIds(tenantId, organizationId);
            queryMap.put("userIds", userIds);
        }
        if (isPtSysadmin) {
            if (null != organizationId && (userIds.size() == FastIotConstants.MagicNumber.ZERO)) {
                userPage = new Page<>();
            } else {
                userPage = baseMapper.getUserPage(userIPage, queryMap);
            }
        } else if (isTenantAdmin) {
            if (null != organizationId && (userIds.size() == FastIotConstants.MagicNumber.ZERO)) {
                userPage = new Page<>();
            } else {
                queryMap.put("isTenantAdmin", true);
                userPage = baseMapper.getUserPage(userIPage, queryMap);
            }
        }
        if (null != userPage) {
            userPage.getRecords().forEach(record -> {
                fillUserStatus(record);
                record.setHasPassword(StringUtils.isNotBlank(record.getActivateToken()));
                record.setPassword(null);
                List<String> list = userOrganizationMappingService.getOrganizationIdsByUserId(record.getId());
                record.setOrganizationIds(list.toArray(new String[list.size()]));
                // 填充部门信息
                List<String> orgNames = userOrganizationMappingService.getOrganizationNamesByUserId(record.getId());
                record.setDepartments(new HashSet<>(orgNames));
                // 填充角色信息
                Set<SysRoleEntity> roles = roleMapper.selectRoleByUserId(record.getId());
                Set<PlainRolesDTO> collect = roles.stream().map(role -> {
                    PlainRolesDTO plainRolesDTO = new PlainRolesDTO();
                    plainRolesDTO.setRoleId(role.getId());
                    plainRolesDTO.setRoleName(role.getName());

                    return plainRolesDTO;
                }).collect(Collectors.toSet());
                record.setRoles(collect);
            });
        }
        return getPageData(userPage, UserDTO.class);
    }

    @Override
    public UserDTO saveAccount(UserDTO userDTO, boolean sendEmail, boolean sendMsg, boolean isTenantAdmin, String tenantId) {
        if (sendMsg && StringUtils.isEmpty(userDTO.getPhoneNumber())) {
            throw new TkDataValidationException("you must specify user phone number if you want send activate email to this user");
        }

        if (sendEmail && StringUtils.isEmpty(userDTO.getEmail())) {
            throw new TkDataValidationException("you must specify user email if you want send activate email to this user");
        }

        SysUserEntity user = new SysUserEntity();
        userDTO.copyToEntity(user, ID, CREATE_TIME, UPDATE_TIME, TENANT_CODE, ACTIVATE_TOKEN);
        boolean userExist;

        if (StringUtils.isNotBlank(userDTO.getPassword())) {
            user.setPassword(passwordEncoder.encode(userDTO.getPassword()));
        } else {
            user.setPassword(passwordEncoder.encode("tfkj028"));
        }

        userExist = baseMapper.selectCount(new QueryWrapper<SysUserEntity>().lambda().eq(SysUserEntity::getUsername, userDTO.getUsername()).eq(SysUserEntity::getTenantId, tenantId)) > 0;

        if (userExist) {
            throw new TkDataValidationException(ErrorMessage.NAME_ALREADY_EXISTS.getMessage());
        } else {
            user.setTenantId(tenantId);
            user.setIsFirst(0);
            user.setLevel(FastIotConstants.LevelValue.IS_PLATFORM_ADMIN);
            user.setCreateTime(new Date());
            baseMapper.insert(user);
            for (String roleId : userDTO.getRoleIds()) {
                // 添加用户角色信息
                SysUserRoleEntity userRole = new SysUserRoleEntity();
                userRole.setUserId(user.getId());
                userRole.setRoleId(roleId);
                userRoleMapper.insert(userRole);
            }
            // 绑定用户和组织的关系
            organizationService.bindUserToOrganization(tenantId, user.getId(), userDTO.getOrganizationIds());
            user.copyToDTO(userDTO, PASSWORD, ACTIVATE_TOKEN);
            return userDTO;
        }
    }

    @Override
    public Optional<UserDTO> getUser(String userId, boolean isPtSysadmin, String tenantId) {
        SysUserEntity user = baseMapper.selectById(userId);
        if (user != null) {
            UserDTO userDTO = new UserDTO();
            user.copyToDTO(userDTO, PASSWORD, ACTIVATE_TOKEN);
            List<String> orgIdList = this.getOrgIdListByUserId(userId);
            if(CollectionUtil.isNotEmpty(orgIdList)){
                userDTO.setDepartments(new HashSet<>(orgIdList));
            }
            if (!isPtSysadmin && !tenantId.equals(user.getTenantId())) {
                return Optional.empty();
            }
            return Optional.of(userDTO);
        } else {
            return Optional.empty();
        }
    }

    @Override
    public List<UserDTO> getUserIdByRealName(List<String> nameList) {
        List<SysUserEntity> sysUserEntities = baseMapper.selectList(Wrappers.lambdaQuery(SysUserEntity.class).in(SysUserEntity::getRealName, nameList));
        if (CollectionUtil.isEmpty(sysUserEntities)) {
            return null;
        }
        List<UserDTO> list = new ArrayList<>(sysUserEntities.size());
        for (SysUserEntity sysUserEntity : sysUserEntities) {
            UserDTO userDTO = new UserDTO();
            sysUserEntity.copyToDTO(userDTO, PASSWORD, ACTIVATE_TOKEN);
            list.add(userDTO);
        }
        return list;
    }

    @Override
    public List<String> getOrgIdListByUserId(String id) {
       List<String> orgIdList =  organizationService.getByUserId(id);
       List<String> result = new ArrayList<>();
        for (String orgId : orgIdList) {
            List<String> childrenId = organizationService.getChildrenId(orgId);
            result.addAll(childrenId);
        }
        return result;
    }

    @Override
    public Optional<List<UserDTO>> getUserByUsername(String username) {

        final List<SysUserEntity> sysUserEntities = baseMapper.selectList(Wrappers.<SysUserEntity>lambdaQuery()
                .eq(SysUserEntity::getUsername, username).or().eq(SysUserEntity::getRealName, username));
        if (sysUserEntities != null) {
            List list = new ArrayList();
            for (SysUserEntity sysUserEntity : sysUserEntities) {
                UserDTO userDTO = new UserDTO();
                sysUserEntity.copyToDTO(userDTO, PASSWORD, ACTIVATE_TOKEN);
                list.add(userDTO);
            }
            return Optional.of(list);

        }

        return Optional.empty();
    }

    @Override
    public UserDTO updateUser(UserDTO userDTO, boolean isPtSysadmin, String tenantId) {
        SysUserEntity user = baseMapper.selectById(userDTO.getId());
//        if (!user.getUsername().equals(userDTO.getUsername())) {
//            throw new TkDataValidationException(ErrorMessage.USERNAME_IS_IMMUTABLE.getMessage());
//        }
        userDTO.setUsername(user.getUsername());
//        validateUserNameAndPhoneNumberAndEmail(userDTO);
        user.setRealName(userDTO.getRealName());
        user.setEnabled(userDTO.isEnabled());
        user.setPhoneNumber(userDTO.getPhoneNumber());
        user.setEmail(userDTO.getEmail());
        user.setAccountExpireTime(userDTO.getAccountExpireTime());
        user.setRemark(userDTO.getRemark());
        user.setUpdateTime(new Date());
        user.setStatus(userDTO.getStatus());
        user.setIsAdmin(userDTO.getIsAdmin());
        if (userDTO.getPassword() != null && !userDTO.getPassword().equals("******")) {
            user.setPassword(passwordEncoder.encode(userDTO.getPassword()));
        }

        userRoleMapper.delete(
                new QueryWrapper<SysUserRoleEntity>()
                        .lambda()
                        .eq(SysUserRoleEntity::getUserId, user.getId()));

        //绑定用户和角色的关系
        if (Objects.nonNull(userDTO.getRoleIds())) {
            for (String roleId : userDTO.getRoleIds()) {
                SysUserRoleEntity userRole = new SysUserRoleEntity();
                userRole.setUserId(user.getId());
                userRole.setRoleId(roleId);
                userRoleMapper.insert(userRole);
            }
        }

        // 绑定用户和组织的关系
        if (Objects.nonNull(userDTO.getOrganizationIds())) {
            organizationService.bindUserToOrganization(tenantId, user.getId(), userDTO.getOrganizationIds());
        }
        baseMapper.updateById(user);
        user.copyToDTO(userDTO, PASSWORD, ACTIVATE_TOKEN);
        return userDTO;
    }

    @Override
    @Transactional
    public boolean deleteUser(Set<String> userIds, boolean isPtSysadmin, String tenantId) {
        if (userIds.contains("81818181-8181-8181-8181-818181818181")) {//超级管理员
            return false;
        }
        userRoleMapper.delete(new LambdaQueryWrapper<SysUserRoleEntity>().in(SysUserRoleEntity::getUserId, userIds));

        userOrganizationMappingMapper.delete(
                new QueryWrapper<TkUserOrganizationMappingEntity>()
                        .lambda()
                        .in(TkUserOrganizationMappingEntity::getUserId, userIds));

        baseMapper.deleteBatchIds(userIds);
        return true;
    }

    @Override
    public UserDTO accountExist(String userName) {
        if (StringUtils.isEmpty(userName)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        UserDTO userDTO = new UserDTO();
        userDTO.setUsername(userName);
        return baseMapper.findUserInfo(userDTO);
    }

    /**
     * 判断手机号是否重复
     *
     * @param phoneNumber 手机号
     * @return
     */
    @Override
    public SysUserEntity findUserByPhoneNumber(String phoneNumber, String userId) {

        if (StringUtils.isEmpty(phoneNumber)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }

        if (userId.equals("")) {
            List<SysUserEntity> list = baseMapper.selectList(new LambdaQueryWrapper<SysUserEntity>().eq(SysUserEntity::getPhoneNumber, phoneNumber));
            return list.size() > 0 ? list.get(0) : null;
        } else {
            List<SysUserEntity> list = baseMapper.selectList(new LambdaQueryWrapper<SysUserEntity>()
                    .eq(SysUserEntity::getPhoneNumber, phoneNumber)
                    .ne(SysUserEntity::getId, userId));
            return list.size() > 0 ? list.get(0) : null;
        }
    }

    @Override
    public SysUserEntity findUserByDingId(String dingId) {
        final SysUserEntity sysUserEntity = baseMapper.selectOne(Wrappers.<SysUserEntity>lambdaQuery().eq(SysUserEntity::getDingId, dingId));
        return sysUserEntity;
    }

    @Override
    public void validateUserNameAndPhoneNumberAndEmail(UserDTO userDTO) {
        if (null == userDTO.getId()) {
            SysUserEntity existUser = baseMapper.selectOne(new LambdaQueryWrapper<SysUserEntity>().eq(SysUserEntity::getUsername, userDTO.getUsername()));
            if (null != existUser) {
                throw new TkDataValidationException(ErrorMessage.USER_NAME_ALREADY_EXISTS.getMessage());
            }
        }
        if (StringUtils.isNotBlank(userDTO.getPhoneNumber())) {
            if (!CHINA_MOBILE_PATTERN.matcher(userDTO.getPhoneNumber()).matches()) {
                throw new TkDataValidationException(ErrorMessage.PROVIDE_CORRECT_PHONE_NUMBER.getMessage());
            }
        }
        if (StringUtils.isNotBlank(userDTO.getEmail())) {
            if (!EMAIL_PATTERN.matcher(userDTO.getEmail()).matches()) {
                throw new TkDataValidationException(ErrorMessage.PROVIDE_CORRECT_EMAIL.getMessage());
            }
        }
        if (StringUtils.isEmpty(userDTO.getEmail()) && StringUtils.isEmpty(userDTO.getPhoneNumber())) {
            throw new TkDataValidationException(ErrorMessage.PHONE_NUMBER_OR_EMAIL_REQUIRED.getMessage());
        }
        boolean needCheck = true;
        if (null != userDTO.getId()) {
            SysUserEntity user = baseMapper.selectById(userDTO.getId());
            if (user.getPhoneNumber() != null && user.getPhoneNumber().equals(userDTO.getPhoneNumber())) {
                needCheck = false;
            }
        }
        if (needCheck) {
            LambdaQueryWrapper<SysUserEntity> queryWrapper = new LambdaQueryWrapper<>();
            if (null != userDTO.getId()) {
                queryWrapper.ne(SysUserEntity::getId, userDTO.getId());
            }
            queryWrapper.and(userLambdaQueryWrapper -> {
                userLambdaQueryWrapper.eq(SysUserEntity::getPhoneNumber, userDTO.getPhoneNumber());
                userLambdaQueryWrapper.or(StringUtils.isNotEmpty(userDTO.getEmail()), (userQueryWrapper -> {
                    userQueryWrapper.eq(SysUserEntity::getEmail, userDTO.getEmail());
                }));
            });
            List<SysUserEntity> existPhoneOrEmail = baseMapper.selectList(queryWrapper);
            if (null != existPhoneOrEmail && existPhoneOrEmail.size() > 0) {
                throw new TkDataValidationException(ErrorMessage.PHONE_OR_EMAIL_HAS_REGISTER.getMessage());
            }
        }
    }

    @Override
    public String[] getUserRoleOrOrganization(RoleOrOrganizationReqDTO roleOrGroupReqDTO) {
        String userId = roleOrGroupReqDTO.getUserId();
        if (StringUtils.isEmpty(userId)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        if (roleOrGroupReqDTO.isQueryRole()) {
            List<String> roleIds = userRoleMapper.selectList(new QueryWrapper<SysUserRoleEntity>().lambda().eq(SysUserRoleEntity::getUserId, userId)).stream().map(SysUserRoleEntity::getRoleId).collect(Collectors.toList());
            if (roleIds == null || roleIds.size() == 0) {
                return null;
            }
            List<String> enableRoles = roleMapper.selectList(new QueryWrapper<SysRoleEntity>().lambda().eq(SysRoleEntity::isEnabled, true).in(roleIds != null && roleIds.size() > 0, SysRoleEntity::getId, roleIds)).stream().map(m -> m.getId()).collect(Collectors.toList());
            if (enableRoles.size() == 0) {
                return null;
            }
            return enableRoles.toArray(new String[enableRoles.size()]);
        }
        if (roleOrGroupReqDTO.isQueryOrganization()) {
            List<String> organizationIds =
                    userOrganizationMappingService.getOrganizationIdsByUserId(userId);
            return organizationIds.toArray(new String[organizationIds.size()]);
        }
        return null;
    }

    @Override
    public List<Map<String, String>> getAddressBook() {
        return baseMapper.getAddressBook();
    }

    @Override
    public void pullUserFromDing(long deptId) {
        final List<SysUserEntity> sysUserEntities = DingUserHandle.pullSysUserEntityFromDing(deptId);

        for (SysUserEntity sysUserEntity : sysUserEntities) {
            if (sysUserEntity.getId() == null) {
                baseMapper.insert(sysUserEntity);
            } else {
                baseMapper.updateById(sysUserEntity);
            }
            //处理用户的组织信息
            String deptId1 = sysUserEntity.getDeptId();
            if (deptId1 != null) {
                deptId1 = deptId1.trim();
                final String[] deptArr
                        = deptId1.replace("[", "").replace("]", "").split(",");

                final List<TkOrganizationEntity> tkOrganizationEntities = organizationMapper.selectList(Wrappers
                        .<TkOrganizationEntity>lambdaQuery().in(TkOrganizationEntity::getDingId, deptArr));

                final List<TkUserOrganizationMappingEntity> tkUserOrganizationMappingEntities = userOrganizationMappingMapper.selectList(Wrappers.
                        <TkUserOrganizationMappingEntity>lambdaQuery().eq(TkUserOrganizationMappingEntity::getUserId, sysUserEntity.getId()));
                final Map<String, TkUserOrganizationMappingEntity> collect =
                        tkUserOrganizationMappingEntities.stream().collect(Collectors.toMap(TkUserOrganizationMappingEntity::getOrganizationId, t -> t));
                List<TkUserOrganizationMappingEntity> list = new ArrayList<>();
                for (TkOrganizationEntity tkOrganizationEntity : tkOrganizationEntities) {
                    if (!collect.containsKey(tkOrganizationEntity.getId())) {
                        //新增
                        TkUserOrganizationMappingEntity tkUserOrganizationMappingEntity = new TkUserOrganizationMappingEntity();
                        tkUserOrganizationMappingEntity.setUserId(sysUserEntity.getId());
                        tkUserOrganizationMappingEntity.setOrganizationId(tkOrganizationEntity.getId());
                        //保存组织映射信息
                        userOrganizationMappingMapper.insert(tkUserOrganizationMappingEntity);
                    }
                }

            }

            //处理用户的角色信息
            //1.查询用户现在是否存在角色  如存在 则不操作   不存在   则新增

            final List<SysUserRoleEntity> sysUserRoleEntities = userRoleMapper.selectList(Wrappers.<SysUserRoleEntity>lambdaQuery()
                    .eq(SysUserRoleEntity::getUserId, sysUserEntity.getId()));
            //没有角色信息  则新增
            if (sysUserRoleEntities == null || sysUserRoleEntities.size() == 0) {
                final SysRoleEntity normalRole = roleMapper.selectOne(Wrappers.<SysRoleEntity>lambdaQuery().eq(SysRoleEntity::getName, "普通用户"));
                SysUserRoleEntity sysUserRoleEntity = new SysUserRoleEntity();
                sysUserRoleEntity.setUserId(sysUserEntity.getId());
                sysUserRoleEntity.setRoleId(normalRole.getId());
                userRoleMapper.insert(sysUserRoleEntity);
            }

        }
    }


}
