package com.tofly.base.utils;

import com.tofly.base.service.miniofile.MinioFileStorageProperties;
import io.minio.*;
import io.minio.messages.Bucket;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Optional;

@Component
@Slf4j
public class MinioUtils {

    private final MinioFileStorageProperties fileStorageProperties;
    private final MinioClient minioClient;
    private final String downloadAddress;
    @Autowired
    public MinioUtils(MinioFileStorageProperties fileStorageProperties, @Value("${file.download.address}") String downloadAddress) {
        this.downloadAddress = downloadAddress;
        log.info("endpoint={}  ,  accessKey={}  ,secretKey={}", fileStorageProperties.getMinioUrl(), fileStorageProperties.getMinioName(), fileStorageProperties.getMinioPass());
        this.fileStorageProperties = fileStorageProperties;
        this.minioClient = MinioClient.builder().credentials(fileStorageProperties.getMinioName(), fileStorageProperties.getMinioPass()).endpoint(fileStorageProperties.getMinioUrl()).build();
        if(bucketExists(fileStorageProperties.getBucketName())) createBucket(fileStorageProperties.getBucketName());
    }
    /**
     * 判断桶是否存在
     */
    @SneakyThrows(Exception.class)
    public boolean bucketExists(String bucketName) {
        return minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
    }


    /**
     * 创建桶
     *
     * @param bucketName 获取全部的桶  minioClient.listBuckets();
     */
    @SneakyThrows(Exception.class)
    public void createBucket(String bucketName) {
        if (!minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build())) {
            minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
        }
    }


    /**
     * 根据bucketName获取信息
     *
     * @param bucketName bucket名称
     */
    @SneakyThrows(Exception.class)
    public Optional<Bucket> getBucket(String bucketName) {
        return minioClient.listBuckets().stream().filter(b -> b.name().equals(bucketName)).findFirst();
    }

    /**
     * 根据bucketName删除信息
     *
     * @param bucketName bucket名称
     */
    @SneakyThrows(Exception.class)
    public void removeBucket(String bucketName) {
        minioClient.removeBucket(RemoveBucketArgs.builder().bucket(bucketName).build());
    }

    /**
     * 获取文件流
     *
     * @param objectName 文件名称
     * @return 二进制流
     */
    @SneakyThrows(Exception.class)
    public InputStream getObject(String objectName) {
        return minioClient.getObject(GetObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(objectName).build());
    }


    /**
     * 上传本地文件
     *
     * @param objectName 对象名称
     * @param fileName   本地文件路径
     */
    @SneakyThrows(Exception.class)
    public ObjectWriteResponse putObject(String objectName, String fileName) {
        return minioClient.uploadObject(UploadObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(objectName).filename(fileName).build());
    }

    /**
     * 通过流上传文件
     *
     * @param objectName  文件对象
     * @param inputStream 文件流
     */
    @SneakyThrows(Exception.class)
    public ObjectWriteResponse putObject(String objectName, InputStream inputStream) {
        return minioClient.putObject(PutObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(objectName).stream(inputStream, inputStream.available(), -1).build());
    }



    /**
     * 下载文件
     * bucketName:桶名
     *
     * @param fileName: 文件名
     */
    @SneakyThrows(Exception.class)
    public void download(String fileName, HttpServletResponse response) {
        // 获取对象的元数据
        StatObjectResponse stat = minioClient.statObject(StatObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(fileName).build());
        response.setContentType(stat.contentType());
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        InputStream is = minioClient.getObject(GetObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(fileName).build());
        IOUtils.copy(is, response.getOutputStream());
        is.close();
    }


    private String getPath(String fileName) {
        return fileStorageProperties
                .getMinioUrl()
                .concat("/")
                .concat(fileStorageProperties.getBucketName())
                .concat("/")
                .concat(fileName);
    }

    public static FileItem createFileItem(InputStream inputStream, String fileName) {
        FileItemFactory factory = new DiskFileItemFactory(16,null);
        String textFieldName = "file";
        FileItem item = factory.createItem(textFieldName, MediaType.APPLICATION_OCTET_STREAM_VALUE,true,fileName);
        int bytesRead = 0;
        byte[] buffer = new byte[10 * 1024 * 1024];
        OutputStream os = null;

        try {
            os = item.getOutputStream();
            while ((bytesRead = inputStream.read(buffer,0,8192)) != -1) {
                os.write(buffer,0,bytesRead);
            }
            inputStream.close();
        } catch (IOException e) {

            throw new IllegalArgumentException("文件上传失败");
        } finally {
            if (os != null) {
                try {
                    //关流
                    os.close();
                } catch (IOException e1) {

                }
            }
            if (inputStream != null) {
                try {
                    //关流
                    inputStream.close();
                } catch (IOException e2) {

                }
            }
        }
        return item;
    }

}