package com.tofly.pms.testmanage.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.exception.BusinessException;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.testmanage.dto.BugInfoBatchDto;
import com.tofly.pms.testmanage.dto.BugUpdateDto;
import com.tofly.pms.testmanage.entity.BugInfo;
import com.tofly.pms.testmanage.entity.BugLog;
import com.tofly.pms.testmanage.mapper.BugInfoMapper;
import com.tofly.pms.testmanage.service.BugInfoService;
import com.tofly.pms.testmanage.service.BugLogService;
import com.tofly.pms.testmanage.service.SystemItemService;
import com.tofly.pms.testmanage.vo.*;
import com.tofly.pms.utils.UserTokenUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 测试BUG表;(TF_PM_BUG_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Slf4j
@Api(tags = "测试BUG表对象功能接口")
@RestController
@RequestMapping("/tfPmBugW")
public class BugInfoController {
    @Autowired
    private BugInfoService bugInfoService;
    @Autowired
    private BugLogService bugLogService;
    @Autowired
    private SystemItemService systemItemService;
    @Autowired
    private ProjectTeamWService projectTeamWService;
    @Autowired
    private BugInfoMapper bugInfoMapper;
    @Autowired
    private ProjectWService projectWService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键testSystemService
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<BugInfo> queryById(Integer id) {
        return ResponseEntity.ok(bugInfoService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param bugInfo 筛选条件
     * @param page    分页对象
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<BugInfo>> paginQuery(BugInfo bugInfo, Page<BugInfo> page) {
        List<Integer> projectIdList = projectWService.getSelfProjectId();
        if (CollectionUtil.isEmpty(projectIdList)) {
            return ResponseEntity.ok(new Page<>());
        }
        String title = bugInfo.getTitle();
        String sureId = bugInfo.getSureId();
        bugInfo.setTitle(null);
        bugInfo.setSureId(null);
        LambdaQueryWrapper<BugInfo> wrapper = new QueryWrapper<>(bugInfo).lambda()
                .like(StringUtils.isNotBlank(title), BugInfo::getTitle, title)
                .eq(BugInfo::getDeleted, 0)
                .in(BugInfo::getProjectId, projectIdList)
                .orderByDesc(BugInfo::getCreateTime);
        if (StringUtils.isNotBlank(sureId)) {
            wrapper.and(wr -> wr.like(BugInfo::getSureId, sureId).or().like(BugInfo::getAlreadySureId, sureId));
        }
        final Page<BugInfo> page1 = bugInfoService.page(page, wrapper);
        List<BugInfo> records = page1.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (BugInfo record : records) {
                Integer functionId = record.getFunctionId();
                String name = systemItemService.getFullNameByFunctionId(functionId);
                record.setFullFunctionName(name);
                String adviseDate = record.getAdviseTime();
                Date startDate = null;
                if (StringUtils.isNotBlank(adviseDate)) {
                    startDate = DateUtil.parse(adviseDate, "yyyy-MM-dd");
                }
                // 0 正常 1 逾期 2 待开始
                Date nowDate = new Date();
                DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
                String format = df.format(nowDate);
                nowDate = DateUtil.parse(format, "yyyy-MM-dd");
                if (Objects.nonNull(startDate) && startDate.before(nowDate)) {
                    record.setTimeStatus(1);
                } else {
                    record.setTimeStatus(0);
                }
            }
        }

        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @param bugInfo 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<BugInfo> add(@RequestBody BugInfo bugInfo) {
        bugInfoService.save(bugInfo);
        return ResponseEntity.ok(bugInfo);
    }

    /**
     * 更新数据
     *
     * @param bugInfo 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PostMapping("updateBugInfo")
    public ResponseEntity<BugInfo> updateBugInfo(@RequestBody BugInfo bugInfo) {
        bugInfoService.updateBugInfo(bugInfo);
        return ResponseEntity.ok(bugInfo);
    }

    /**
     * 更新数据
     *
     * @param bugInfo 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<BugInfo> edit(@RequestBody BugInfo bugInfo) {
        bugInfoService.updateById(bugInfo);
        BugLog bugLog = new BugLog();
        bugLog.setBugId(bugInfo.getId());
        bugLog.setStatus(bugInfo.getStatus());
        bugLog.setJsonData(JSON.toJSONString(bugInfo));
        bugLogService.save(bugLog);
        return ResponseEntity.ok(bugInfo);
    }

    @ApiOperation("批量修改")
    @PutMapping("updateBatch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody BugInfoBatchDto bugInfoBatchDto) {
        final List<BugInfo> datas = bugInfoBatchDto.getDatas();
        for (BugInfo data : datas) {
            Integer id = data.getId();
            BugInfo bugInfo = bugInfoService.getById(id);
            bugInfo.setSureId(data.getSureId());
            bugInfo.setHeadId(data.getHeadId());
            bugInfo.setStatus(data.getStatus());
            bugInfoService.updateBugInfo(bugInfo);
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 批量修改或者保存
     *
     * @param bugInfoBatchDto 实例对象
     * @return 实例对象
     */
    @ApiOperation("批量修改或者保存")
    @PutMapping("saveOrUpdateBatch")
    public ResponseEntity<Boolean> saveOrUpdateBatch(@RequestBody BugInfoBatchDto bugInfoBatchDto) {
        final List<BugInfo> datas = bugInfoBatchDto.getDatas();
        bugInfoService.saveOrUpdateBatch(datas);
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 只更新负责人和确认人
     * 0
     *
     * @param bugUpdateDto 实例对象
     * @return 实例对象
     */
    @ApiOperation("只更新负责人和确认人")
    @PutMapping("saveOrUpdateBatchUser")
    public ResponseEntity<Boolean> saveOrUpdateBatchUser(@RequestBody BugUpdateDto bugUpdateDto) {
        final String bugIds = bugUpdateDto.getBugIds();
        final List<Integer> collect = Arrays.asList(bugIds.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        final List<BugInfo> list = bugInfoService.list(Wrappers.<BugInfo>lambdaQuery().in(BugInfo::getId, collect));
        final List<BugInfo> collect1 = list.stream().map(obj -> {
            obj.setHeadId(bugUpdateDto.getHeadId());
            obj.setSureId(bugUpdateDto.getSureId());
            return obj;
        }).collect(Collectors.toList());
        bugInfoService.saveOrUpdateBatch(collect1);
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    @Transactional(rollbackFor = Exception.class)
    public ResponseEntity<Boolean> deleteById(String ids) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "用户不合法");
        final String[] split = ids.split(",");
        for (String s : split) {
            Integer id = Integer.valueOf(s);
            BugInfo byId = bugInfoService.getById(id);
            if (Objects.isNull(byId)) {
                continue;
            }
            if (!Objects.equals(userID, byId.getCreateUser())) {
                Integer projectId = byId.getProjectId();
                ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                        .eq(ProjectTeamW::getProjectId, projectId)
                        .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                                .or()
                                .like(ProjectTeamW::getTestId, userID)));
                if (Objects.isNull(one)) {
                    RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                    Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                    if (!has) {
                        throw new BusinessException("没有权限");
                    }
                }
            }
            byId.setDeleted(1);
            bugInfoMapper.updateById(byId);
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * allType
     * 0:查询全部
     * 1：查询待处理
     * 2：查询被驳回
     *
     * @param bugInfo
     * @param page
     * @param allType
     * @return
     */
    @ApiOperation("待我处理")
    @GetMapping("/toResolve")
    public ResponseEntity<Page<BugInfo>> getToResolve(BugInfo bugInfo, Page<BugInfo> page, Integer allType) {
        final String tokenUserID = UserTokenUtil.getTokenUserID();
        String severity = bugInfo.getSeverity();
        String urgency = bugInfo.getUrgency();
        Integer systemId = bugInfo.getSystemId();
        String title = bugInfo.getTitle();
        bugInfo.setSeverity(null);
        bugInfo.setUrgency(null);
        bugInfo.setSystemId(null);
        bugInfo.setTitle(null);
        bugInfo.setShortDescription(null);
        bugInfo.setDescription(null);
        Page<BugInfo> page1;
        LambdaQueryWrapper<BugInfo> wrapper = Wrappers
                .lambdaQuery(bugInfo)
                .eq(BugInfo::getDeleted, 0)
                .like(BugInfo::getHeadId, tokenUserID)
                .eq(StringUtils.isNotBlank(severity), BugInfo::getSeverity, severity)
                .eq(StringUtils.isNotBlank(urgency), BugInfo::getUrgency, urgency)
                .eq(Objects.nonNull(systemId), BugInfo::getSystemId, systemId);
        if (StringUtils.isNotBlank(title)) {
            wrapper.and(w -> w.like(BugInfo::getTitle, title)
                    .or().like(BugInfo::getShortDescription, title)
                    .or().like(BugInfo::getDescription, title)
            );
        }

        if (Objects.isNull(allType) || allType == 0) {
            wrapper.and(w -> w.eq(BugInfo::getStatus, "1")
                    .or()
                    .eq(BugInfo::getStatus, "4"));
            page1 = bugInfoService.page(page,
                    wrapper
            );
        } else if (allType == 1) {
            wrapper.eq(BugInfo::getStatus, "1");
            page1 = bugInfoService.page(page, wrapper);
        } else {
            wrapper.eq(BugInfo::getStatus, "4");
            page1 = bugInfoService.page(page, wrapper);

        }
        List<BugInfo> records = page1.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (BugInfo record : records) {
                Integer functionId = record.getFunctionId();
                String name = systemItemService.getFullNameByFunctionId(functionId);
                record.setFullFunctionName(name);

                String adviseDate = record.getAdviseTime();
                Date startDate = null;
                if (StringUtils.isNotBlank(adviseDate)) {
                    startDate = DateUtil.parse(adviseDate, "yyyy-MM-dd");
                }
                // 0 正常 1 逾期 2 待开始
                Date nowDate = new Date();
                DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
                String format = df.format(nowDate);
                nowDate = DateUtil.parse(format, "yyyy-MM-dd");
                if (Objects.nonNull(startDate) && startDate.before(nowDate)) {
                    record.setTimeStatus(1);
                } else {
                    record.setTimeStatus(0);
                }
            }
        }
        return ResponseEntity.ok(page1);
    }

    @ApiOperation("待我处理tab统计")
    @GetMapping("/toResolveCount")
    public ResponseEntity<ToResolveCountVo> gettoResolveCount(BugInfo bugInfo) {
        final ToResolveCountVo toResolveCountVo = bugInfoService.gettoResolveCount(bugInfo);
        return ResponseEntity.ok(toResolveCountVo);
    }

    @ApiOperation("待我处理左侧统计")
    @GetMapping("/toResolveCountLeft")
    public ResponseEntity<List<BugStatusCountProjectVo>> toResolveCountLeft(BugInfo bugInfo) {
        bugInfo.setHeadId(UserTokenUtil.getTokenUserID());
        final List<BugStatusCountProjectVo> bugStatusCountProjectVos = bugInfoService.BugStatusCountOverProject(bugInfo);
        return ResponseEntity.ok(bugStatusCountProjectVos);
    }


    @ApiOperation("待我验证左侧统计")
    @GetMapping("/toSureCountLeft")
    public ResponseEntity<List<ToSureLeftVo>> toSureCountLeft(BugInfo bugInfo) {


        bugInfo.setHeadId(UserTokenUtil.getTokenUserID());
        final List<ToSureLeftVo> toSureLeftVos = bugInfoService.toSureCountLeft(UserTokenUtil.getTokenUserID());
        return ResponseEntity.ok(toSureLeftVos);
    }

    /**
     * @param allType 查询类型
     *                1：全部
     *                2：初次
     *                3：驳回
     * @param bugInfo
     * @param page
     * @return
     */
    @ApiOperation("待我验证")
    @GetMapping("/toSure")
    public ResponseEntity<Page<BugInfo>> gettoSure(BugInfo bugInfo,
                                                   Page<BugInfo> page,
                                                   int allType,
                                                   String keyword,
                                                   String startDate,
                                                   String endDate) {
        final String tokenUserID = UserTokenUtil.getTokenUserID();
        List<Integer> bugIdList = null;
        if (allType > 1) {
            bugIdList = bugLogService.getBugIdList(allType);
            //如果查询出的没有人员  则需要设置特殊值 避免条件失效
            if (bugIdList.isEmpty()) {
                bugIdList.add(-1);
            }
        }
        LambdaQueryWrapper<BugInfo> wrapper = Wrappers.lambdaQuery(bugInfo)
                .eq(BugInfo::getDeleted, 0)
                .like(BugInfo::getSureId, tokenUserID)
                .eq(BugInfo::getStatus, "2")
                .in(CollectionUtil.isNotEmpty(bugIdList), BugInfo::getId, bugIdList);
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            try {
                DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Date date1 = df.parse(startDate);
                Date date2 = df.parse(endDate);
                wrapper.between(BugInfo::getCompleteTime, date1, date2);
            } catch (ParseException e) {
                log.info("时间格式错误");
                throw new RuntimeException("时间格式错误");
            }
        }
        if (StringUtils.isNotBlank(keyword)) {
            wrapper.and(wr -> wr.like(BugInfo::getTitle, keyword)
                    .or()
                    .eq(BugInfo::getCode, keyword));
        }
        final Page<BugInfo> page1 = bugInfoService.page(page, wrapper);
        List<BugInfo> records = page1.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (BugInfo record : records) {
                Integer functionId = record.getFunctionId();
                String name = systemItemService.getFullNameByFunctionId(functionId);
                record.setFullFunctionName(name);
            }
        }
        return ResponseEntity.ok(page1);
    }

    /**
     * * @param allType 查询类型
     * *                1：全部
     * *                2：初次
     * *                3：驳回
     *
     * @param bugInfo
     * @return
     */
    @ApiOperation("待我验证统计")
    @GetMapping("/toSureCount")
    public ResponseEntity<Map<String, Integer>> gettoSureCount(BugInfo bugInfo, String startDate, String endDate) {
        final String tokenUserID = UserTokenUtil.getTokenUserID();
        Date date1 = null;
        Date date2 = null;
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            try {
                DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                date1 = df.parse(startDate);
                date2 = df.parse(endDate);
            } catch (ParseException e) {
                log.info("时间格式错误");
                throw new RuntimeException("时间格式错误");
            }
        }
        List<Integer> bugIdList2 = bugLogService.getBugIdList(2);
        if (CollectionUtil.isEmpty(bugIdList2)) {
            bugIdList2.add(-1);
        }
        List<Integer> bugIdList3 = bugLogService.getBugIdList(3);
        if (CollectionUtil.isEmpty(bugIdList3)) {
            bugIdList3.add(-1);
        }
        List<BugInfo> list1 = bugInfoService.list(Wrappers.lambdaQuery(bugInfo)
                .like(BugInfo::getSureId, tokenUserID)
                .eq(BugInfo::getDeleted, 0)
                .eq(BugInfo::getStatus, "2")
                .between(Objects.nonNull(date1), BugInfo::getCompleteTime, date1, date2));
        List<BugInfo> list2 = bugInfoService.list(Wrappers.lambdaQuery(bugInfo)
                .like(BugInfo::getSureId, tokenUserID)
                .eq(BugInfo::getStatus, "2")
                .eq(BugInfo::getDeleted, 0)
                .in(BugInfo::getId, bugIdList2)
                .between(Objects.nonNull(date1), BugInfo::getCompleteTime, date1, date2));
        List<BugInfo> list3 = bugInfoService.list(Wrappers.lambdaQuery(bugInfo)
                .like(BugInfo::getSureId, tokenUserID)
                .eq(BugInfo::getDeleted, 0)
                .eq(BugInfo::getStatus, "2")
                .in(BugInfo::getId, bugIdList3)
                .between(Objects.nonNull(date1), BugInfo::getCompleteTime, date1, date2));
        Map<String, Integer> res = new LinkedHashMap<>(3);
        res.put("all", list1.size());
        res.put("first", list2.size());
        res.put("more", list3.size());
        return ResponseEntity.ok(res);
    }

    @ApiOperation("缺陷状态数量统计")
    @GetMapping("/bugStatusCount")
    public ResponseEntity<BugStatusCountWithAllVo> bugStatusCount(BugInfo bugInfo) {
        List<Integer> selfProjectId = projectWService.getSelfProjectId();
        if (CollectionUtil.isEmpty(selfProjectId)) {
            return ResponseEntity.ok(new BugStatusCountWithAllVo());
        }
        String title = bugInfo.getTitle();
        String sureId = bugInfo.getSureId();
        bugInfo.setTitle(null);
        bugInfo.setSureId(null);
        BugStatusCountWithAllVo b = new BugStatusCountWithAllVo();
        LambdaQueryWrapper<BugInfo> w = Wrappers.query(bugInfo)
                .lambda()
                .eq(BugInfo::getDeleted, 0)
                .like(StringUtils.isNotBlank(title),
                        BugInfo::getTitle, title)
                .in(BugInfo::getProjectId, selfProjectId);
        if (StringUtils.isNotBlank(sureId)) {
            w.and(wr -> wr.like(BugInfo::getSureId, sureId)
                    .or()
                    .like(BugInfo::getAlreadySureId, sureId));
        }
        final long count = bugInfoService.count(w);
        b.setTotal(count);
        bugInfo.setTitle(title);
        bugInfo.setSureId(sureId);
        final List<BugStatusCountVo> bugStatusCountVos = bugInfoService.bugStatusOverCount(bugInfo);
        b.setStatusCount(bugStatusCountVos);
        return ResponseEntity.ok(b);
    }


    @ApiOperation("导出excel")
    @GetMapping("/export")
    public void export(@RequestParam(name = "projectId", required = false) Integer projectId,
                       @RequestParam(name = "systemId", required = false) Integer systemId,
                       @RequestParam(name = "headId", required = false) String headId,
                       @RequestParam(name = "createUser", required = false) String createUser) throws IOException {
        bugInfoService.export(projectId, systemId, headId, createUser);
    }

    @ApiOperation("获取版本号")
    @GetMapping("/getVersionByProjectId")
    public ResponseEntity<LinkedHashSet<String>> getVersionByProjectId(@RequestParam("projectId") Integer projectId) {
        List<String> versionList = bugInfoService.listObjs(Wrappers.lambdaQuery(BugInfo.class)
                        .eq(BugInfo::getProjectId, projectId)
                        .select(BugInfo::getVersion),
                item -> {
                    if (item instanceof String) {
                        return (String) item;
                    }
                    throw new RuntimeException("类型转换错误");
                });
        return ResponseEntity.ok(new LinkedHashSet<>(versionList));
    }
}