package com.tofly.pms.utils;

import cn.hutool.extra.spring.SpringUtil;
import com.tofly.pms.config.MinioFileStorageProperties;
import com.tofly.pms.entity.dto.FileUploadResponse;
import com.tofly.pms.file.entity.File;
import com.tofly.pms.file.service.FileService;
import io.minio.*;
import io.minio.http.Method;
import io.minio.messages.Bucket;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Component
@Slf4j
public class MinioUtils {

    private final MinioFileStorageProperties fileStorageProperties;
    private final MinioClient minioClient;
    private final String downloadAddress;
    @Autowired
    public MinioUtils(MinioFileStorageProperties fileStorageProperties, @Value("${file.download.address}") String downloadAddress) {
        this.downloadAddress = downloadAddress;
        log.info("endpoint={}  ,  accessKey={}  ,secretKey={}", fileStorageProperties.getMinioUrl(), fileStorageProperties.getMinioName(), fileStorageProperties.getMinioPass());
        this.fileStorageProperties = fileStorageProperties;
        this.minioClient = MinioClient.builder().credentials(fileStorageProperties.getMinioName(), fileStorageProperties.getMinioPass()).endpoint(fileStorageProperties.getMinioUrl()).build();
        if(bucketExists(fileStorageProperties.getBucketName())) createBucket(fileStorageProperties.getBucketName());
    }
    /**
     * 判断桶是否存在
     */
    @SneakyThrows(Exception.class)
    public boolean bucketExists(String bucketName) {
        return minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
    }


    /**
     * 创建桶
     *
     * @param bucketName 获取全部的桶  minioClient.listBuckets();
     */
    @SneakyThrows(Exception.class)
    public void createBucket(String bucketName) {
        if (!minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build())) {
            minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
        }
    }


    /**
     * 根据bucketName获取信息
     *
     * @param bucketName bucket名称
     */
    @SneakyThrows(Exception.class)
    public Optional<Bucket> getBucket(String bucketName) {
        return minioClient.listBuckets().stream().filter(b -> b.name().equals(bucketName)).findFirst();
    }

    /**
     * 根据bucketName删除信息
     *
     * @param bucketName bucket名称
     */
    @SneakyThrows(Exception.class)
    public void removeBucket(String bucketName) {
        minioClient.removeBucket(RemoveBucketArgs.builder().bucket(bucketName).build());
    }

    /**
     * 获取文件流
     *
     * @param objectName 文件名称
     * @return 二进制流
     */
    @SneakyThrows(Exception.class)
    public InputStream getObject(String objectName) {
        return minioClient.getObject(GetObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(objectName).build());
    }


    /**
     * 上传本地文件
     *
     * @param objectName 对象名称
     * @param fileName   本地文件路径
     */
    @SneakyThrows(Exception.class)
    public ObjectWriteResponse putObject(String objectName, String fileName) {
        return minioClient.uploadObject(UploadObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(objectName).filename(fileName).build());
    }

    /**
     * 通过流上传文件
     *
     * @param objectName  文件对象
     * @param inputStream 文件流
     */
    @SneakyThrows(Exception.class)
    public ObjectWriteResponse putObject(String objectName, InputStream inputStream) {
        return minioClient.putObject(PutObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(objectName).stream(inputStream, inputStream.available(), -1).build());
    }


    /**
     * 單個文件上傳遞
     *
     * @param multipartFile
     * @return
     */
    @SneakyThrows(Exception.class)
    public FileUploadResponse upload(MultipartFile multipartFile) {
        String fileName = multipartFile.getOriginalFilename();
        String[] split = fileName.split("\\.");
        if (split.length > 1) {
            fileName = split[0] + "_" + System.currentTimeMillis() + "." + split[1];
        } else {
            fileName = fileName + System.currentTimeMillis();
        }
        String staticPath = getPath(fileName);
        String realDownloadPath = null;
        InputStream in = null;
        try {
            in = multipartFile.getInputStream();
            minioClient.putObject(PutObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(fileName).stream(in, in.available(), -1).contentType(multipartFile.getContentType()).build());
            getUploadObjectUrl(fileName, 7 * 24 * 60 * 60);
            try {
                // 获取request
                if(null != RequestContextHolder.getRequestAttributes()){
                    HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
                    // 生成下载接口的地址
                  //  realDownloadPath = request.getRequestURL().toString().replace(request.getServletPath(), "/commonFile/" + fileName);
                    realDownloadPath = downloadAddress + "/commonFile/" + fileName;
                }else{
                    realDownloadPath = downloadAddress + "/commonFile/" + fileName;
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.info("Could not upload file " + fileName + ". Please try again!");
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
         File file = new File();
        file.setFileName(multipartFile.getOriginalFilename());
        file.setFilePath(realDownloadPath);
        file.setOtherName(fileName);
        file.setFileSize(String.valueOf(multipartFile.getSize()));
        final FileService bean = SpringUtil.getBean(FileService.class);
        bean.save(file);

        return new FileUploadResponse(fileName, realDownloadPath, multipartFile.getContentType(), multipartFile.getSize(), staticPath,file.getId(),null);
    }


    /**
     * description: 上传文件
     *
     * @param multipartFile
     * @return: java.lang.String
     */
    public List<FileUploadResponse> upload(List<MultipartFile> multipartFile) {
        List<FileUploadResponse> names = new ArrayList<>();
        for (MultipartFile file : multipartFile) {
            try {
                String fileName = file.getOriginalFilename();
                FileUploadResponse upload = upload(file);
                names.add(upload);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return names;
    }


    /**
     * 获取文件外链
     *
     * @param objectName 文件名称
     * @param expires    过期时间 <=7 秒级
     * @return url
     */
    @SneakyThrows(Exception.class)
    public String getUploadObjectUrl(String objectName, Integer expires) {
        return minioClient.getPresignedObjectUrl(GetPresignedObjectUrlArgs.builder().method(Method.PUT).bucket(fileStorageProperties.getBucketName()).object(objectName).expiry(expires).build());
    }

    /**
     * 下载文件
     * bucketName:桶名
     *
     * @param fileName: 文件名
     */
    @SneakyThrows(Exception.class)
    public void download(String fileName, HttpServletResponse response) {
        // 获取对象的元数据
        StatObjectResponse stat = minioClient.statObject(StatObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(fileName).build());
        response.setContentType(stat.contentType());
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Content-Disposition", "filename=" + URLEncoder.encode(fileName, "UTF-8"));
        InputStream is = minioClient.getObject(GetObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(fileName).build());
        IOUtils.copy(is, response.getOutputStream());
        is.close();
    }
    /**
     * 文件预览
     * bucketName:桶名
     *
     * @param fileName: 文件名
     */
    @SneakyThrows(Exception.class)
    public void downloadView(String fileName, HttpServletResponse response) {
        // 获取对象的元数据
        StatObjectResponse stat = minioClient.statObject(StatObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(fileName).build());
        response.setContentType(stat.contentType());
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        InputStream is = minioClient.getObject(GetObjectArgs.builder().bucket(fileStorageProperties.getBucketName()).object(fileName).build());
        IOUtils.copy(is, response.getOutputStream());
        is.close();
    }

    public FileUploadResponse saveInputStream(String videoUrl, String fileName) {
        try (InputStream inputStream = new URL(videoUrl).openStream()) {
            if(inputStream.available() == 0){
                return null;
            }
            FileItem fileItem = createFileItem(inputStream, fileName);
            MultipartFile commonsMultipartFile = new CommonsMultipartFile(fileItem);
            FileUploadResponse upload = upload(commonsMultipartFile);
            return upload;
        } catch (Throwable e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    private String getPath(String fileName) {
        return fileStorageProperties
                .getMinioUrl()
                .concat("/")
                .concat(fileStorageProperties.getBucketName())
                .concat("/")
                .concat(fileName);
    }

    public static FileItem createFileItem(InputStream inputStream, String fileName) {
        FileItemFactory factory = new DiskFileItemFactory(16,null);
        String textFieldName = "file";
        FileItem item = factory.createItem(textFieldName, MediaType.APPLICATION_OCTET_STREAM_VALUE,true,fileName);
        int bytesRead = 0;
        byte[] buffer = new byte[10 * 1024 * 1024];
        OutputStream os = null;

        try {
            os = item.getOutputStream();
            while ((bytesRead = inputStream.read(buffer,0,8192)) != -1) {
                os.write(buffer,0,bytesRead);
            }
            inputStream.close();
        } catch (IOException e) {

            throw new IllegalArgumentException("文件上传失败");
        } finally {
            if (os != null) {
                try {
                    //关流
                    os.close();
                } catch (IOException e1) {

                }
            }
            if (inputStream != null) {
                try {
                    //关流
                    inputStream.close();
                } catch (IOException e2) {

                }
            }
        }
        return item;
    }

}