package com.tofly.pms.working.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.testmanage.entity.BugInfo;
import com.tofly.pms.working.entity.PerformanceAppraisal;
import com.tofly.pms.working.entity.SimpleUser;
import com.tofly.pms.working.entity.TaskW;
import com.tofly.pms.working.mapper.PerformanceAppraisalMapper;
import com.tofly.pms.working.service.PerformanceAppraisalService;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.swing.text.DateFormatter;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author sheldon
 * @description 针对表【tf_pm_performance_appraisal_w(研发绩效考核统计)】的数据库操作Service实现
 * @createDate 2024-06-26 11:03:27
 */
@Transactional(rollbackFor = Exception.class)
@Service
public class PerformanceAppraisalServiceImpl extends ServiceImpl<PerformanceAppraisalMapper, PerformanceAppraisal>
        implements PerformanceAppraisalService {
    private static final Pattern month_pattern = Pattern.compile("^20\\d{2}-(0[1-9]|1[012])$");
    private static final SimpleDateFormat date_pattern = new SimpleDateFormat("yyyy-MM-dd");

    /**
     * -- 考核月份 month
     * -- 部门 department
     * -- 姓名  username
     * -- 项目数量 project_count
     * -- 任务总数 task_count
     * -- 任务处理数量 task_done_count
     * -- 任务驳回次数 task_reject_count
     * -- 任务逾期数量 task_overdue_count
     * -- 缺陷总数 bug_count
     * -- 缺陷处理数量 bug_done_count
     * -- 缺陷驳回次数 bug_reject_count
     * -- 缺陷逾期数量 bug_overdue_count
     * -- 预计工作量（天）predict_work_time
     * -- 实际工作量（天）fact_work_time
     * -- 驳回率
     * -- 逾期率
     * -- 任务相关项目 task_project_name
     * -- 缺陷相关项目 bug_project_name
     *
     * @param month 月份
     * @return 成功、失败
     * 1、根据当前时间，确定要统计的月份（例如：2024-05）
     * 2、查询用户表，部门表，确定有多少人需要统计（A部门-张三，B部门-李四）
     * 3、根据用户id，查询用户在任务表里面在当前月份的任务，统计出：任务总数，任务处理完成数量，预计工作量，实际工作量，任务相关的项目
     * 4、根据用户id，查询用户在任务日志表里面在当前月份的任务驳回次数，任务逾期数量
     * 5、缺陷表
     * 6、缺陷日志表
     */

    public Boolean automaticStatistics(String month) {
        if (StringUtils.isBlank(month)) {
            LocalDate localDate = LocalDate.now().minusMonths(1);
            month = localDate.format(DateTimeFormatter.ofPattern("yyyy-MM"));
        } else {
            Assert.isTrue(month_pattern.matcher(month).matches(), "月份格式不正确");
        }
        this.remove(Wrappers.lambdaQuery(PerformanceAppraisal.class)
                .likeRight(PerformanceAppraisal::getMonth, month));
        List<SimpleUser> userList = baseMapper.getUserList();
        if (CollectionUtil.isEmpty(userList)) {
            return Boolean.TRUE;
        }
        for (SimpleUser user : userList) {
            PerformanceAppraisal appraisal = getPerformanceAppraisal(user, month);
            this.save(appraisal);
        }
        return Boolean.TRUE;
    }

    private PerformanceAppraisal getPerformanceAppraisal(SimpleUser user, String month) {
        PerformanceAppraisal performanceAppraisal = new PerformanceAppraisal();
        performanceAppraisal.setMonth(month);
        performanceAppraisal.setUsername(user.getUsername());
        performanceAppraisal.setDepartment(user.getDepartment());
        String userId = user.getId();
        performanceAppraisal.setUserId(userId);
        Set<Integer> projectIdSet = new HashSet<>();
        //根据用户id，查询用户在任务表里负责的任务
        List<TaskW> taskList = baseMapper.getTaskList(userId, month);
        if (CollectionUtil.isEmpty(taskList)) {
            performanceAppraisal.setTaskCount(0);
            performanceAppraisal.setTaskDoneCount(0L);
            performanceAppraisal.setTaskOverdueCount(0L);
            performanceAppraisal.setTaskRejectCount(0L);
            performanceAppraisal.setTaskProjectName(null);

        } else {
            // 如果任务不为0，设置任务相关项
            performanceAppraisal.setTaskCount(taskList.size());
            performanceAppraisal.setTaskDoneCount(taskList.stream().filter(taskW -> "3".equals(taskW.getStatus())).count());
            List<Integer> taskIdList = taskList.stream().map(TaskW::getId).collect(Collectors.toList());
            performanceAppraisal.setTaskOverdueCount(taskList.stream().filter(taskW -> {
                String planEndDate = taskW.getPlanEndDate();
                Date completeTime = taskW.getCompleteTime();
                if (Objects.isNull(completeTime)) {
                    return true;
                }
                String completeDate = date_pattern.format(completeTime);
                return planEndDate.compareTo(completeDate) < 0;
            }).count());
            Long rejectCount = baseMapper.getRejectCount(taskIdList);
            performanceAppraisal.setTaskRejectCount(rejectCount);
            List<Integer> projectIdList = taskList.stream().map(TaskW::getProjectId).distinct().collect(Collectors.toList());
            List<String> projectNameList = baseMapper.getProjectNameList(projectIdList);
            performanceAppraisal.setTaskProjectName(String.join(",", projectNameList));
            projectIdSet.addAll(projectIdList);
            Double predictWorkTime= taskList.stream().map(TaskW::getPredictWorkload).filter(Objects::nonNull).reduce(0D,Double::sum);
            performanceAppraisal.setPredictWorkTime(String.format("%.2f",predictWorkTime/8.0));
            Double factWorkTime = taskList.stream().map(TaskW::getTime).filter(Objects::nonNull).reduce(0D, Double::sum);
            performanceAppraisal.setFactWorkTime(String.format("%.2f", factWorkTime/8.0));
        }
        //根据用户id，查询用户在缺陷表里面负责的缺陷
        List<BugInfo> bugList = baseMapper.getBugList(userId, month);
        if (CollectionUtil.isEmpty(bugList)) {
            performanceAppraisal.setBugCount(0);
            performanceAppraisal.setBugDoneCount(0L);
            performanceAppraisal.setBugOverdueCount(0L);
            performanceAppraisal.setBugRejectCount(0L);
            performanceAppraisal.setBugProjectName(null);
        } else {
            performanceAppraisal.setBugCount(bugList.size());
            performanceAppraisal.setBugDoneCount(bugList.stream().filter(bugInfo -> "3".equals(bugInfo.getStatus())).count());
            performanceAppraisal.setBugOverdueCount(bugList.stream().filter(bugInfo -> {
                String adviseTime = bugInfo.getAdviseTime();
                Date completeTime = bugInfo.getCompleteTime();
                if (Objects.isNull(completeTime)) {
                    return true;
                }
                String completeDate = date_pattern.format(completeTime);
                return adviseTime.compareTo(completeDate) < 0;
            }).count());
            List<Integer> bugIdList = bugList.stream().map(BugInfo::getId).collect(Collectors.toList());
            Long rejectCount = baseMapper.getRejectCount2(bugIdList);
            performanceAppraisal.setBugRejectCount(rejectCount);
            List<Integer> projectIdList = bugList.stream().map(BugInfo::getProjectId).distinct().collect(Collectors.toList());
            List<String> projectNameList = baseMapper.getProjectNameList(projectIdList);
            performanceAppraisal.setBugProjectName(String.join(",", projectNameList));
            projectIdSet.addAll(projectIdList);
        }
        performanceAppraisal.setProjectCount(projectIdSet.size());
        return performanceAppraisal;
    }

    /**
     * 每月1号23点执行的任务
     */
    @Scheduled(cron = "0 0 23 1 * ?")
    public void doScheduling(){
        LocalDate localDate = LocalDate.now().minusMonths(1);
       String month = localDate.format(DateTimeFormatter.ofPattern("yyyy-MM"));
        this.automaticStatistics(month);
    }
}




